@@ -1,4 +1,4 @@
-# vim: tabstop = 4 shiftwidth=4 softtabstop=4
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 # Copyright 2010 United States Government as represented by the
 # Administrator of the National Aeronautics and Space Administration.
@@ -19,6 +19,7 @@
 
 import webob.exc
 from heat.common import wsgi
+import heat.openstack.common.rpc.common as rpc_common
 
 
 class HeatAPIException(webob.exc.HTTPError):
@@ -204,6 +205,15 @@ class HeatThrottlingError(HeatAPIException):
     explanation = "Request was denied due to request throttling"
 
 
+class AlreadyExistsError(HeatAPIException):
+    '''
+    Resource with the name requested already exists
+    '''
+    code = 400
+    title = 'AlreadyExists'
+    explanation = "Resource with the name requested already exists"
+
+
 # Not documented in the AWS docs, authentication failure errors
 class HeatAccessDeniedError(HeatAPIException):
     '''
@@ -250,17 +260,29 @@ def map_remote_error(ex):
             'StackNotFound',
             'ResourceNotFound',
             'ResourceNotAvailable',
+            'ResourceTypeNotFound',
             'PhysicalResourceNotFound',
             'WatchRuleNotFound',
-            'StackExists',
             'StackValidationFailed',
+            'InvalidTemplateReference',
+            'UnknownUserParameter',
+            'UserParameterMissing',
+            'InvalidTemplateParameter',
         )
         denied_errors = ('Forbidden', 'NotAuthorized')
+        already_exists_errors = ('StackExists')
 
-        if ex.exc_type in inval_param_errors:
-            return HeatInvalidParameterValueError(detail=ex.value)
-        elif ex.exc_type in denied_errors:
-            return HeatAccessDeniedError(detail=ex.value)
+        ex_type = ex.__class__.__name__
+
+        if ex_type.endswith(rpc_common._REMOTE_POSTFIX):
+            ex_type = ex_type[:-len(rpc_common._REMOTE_POSTFIX)]
+
+        if ex_type in inval_param_errors:
+            return HeatInvalidParameterValueError(detail=str(ex.message))
+        elif ex_type in denied_errors:
+            return HeatAccessDeniedError(detail=str(ex.message))
+        elif ex_type in already_exists_errors:
+            return AlreadyExistsError(detail=str(ex.message))
         else:
             # Map everything else to internal server error for now
-            return HeatInternalFailureError(detail=ex.value)
+            return HeatInternalFailureError(detail=str(ex.message))
