@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -11,63 +13,62 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-"""Stack endpoint for Heat CloudFormation v1 API."""
+"""
+Stack endpoint for Heat CloudFormation v1 API.
+"""
 
+import json
 import socket
 
-from oslo_log import log as logging
-from oslo_serialization import jsonutils
-
 from heat.api.aws import exception
 from heat.api.aws import utils as api_utils
-from heat.common import exception as heat_exception
-from heat.common.i18n import _
-from heat.common import identifier
-from heat.common import policy
-from heat.common import template_format
-from heat.common import urlfetch
 from heat.common import wsgi
-from heat.rpc import api as rpc_api
+from heat.common import exception as heat_exception
 from heat.rpc import client as rpc_client
+from heat.common import template_format
+from heat.rpc import api as engine_api
+from heat.common import identifier
+from heat.common import urlfetch
+from heat.common import policy
 
-LOG = logging.getLogger(__name__)
+from heat.openstack.common import log as logging
+from heat.openstack.common.gettextutils import _
+
+logger = logging.getLogger(__name__)
 
 
 class StackController(object):
 
-    """WSGI controller for stacks resource in Heat CloudFormation v1 API.
-
-    Implements the API actions.
+    """
+    WSGI controller for stacks resource in Heat CloudFormation v1 API
+    Implements the API actions
     """
 
     def __init__(self, options):
         self.options = options
-        self.rpc_client = rpc_client.EngineClient()
+        self.engine_rpcapi = rpc_client.EngineClient()
         self.policy = policy.Enforcer(scope='cloudformation')
 
-    def default(self, req, **args):
-        raise exception.HeatInvalidActionError()
-
     def _enforce(self, req, action):
-        """Authorize an action against the policy.json and policies in code."""
+        """Authorize an action against the policy.json."""
         try:
-            self.policy.enforce(req.context, action, is_registered_policy=True)
+            self.policy.enforce(req.context, action, {})
         except heat_exception.Forbidden:
-            msg = _('Action %s not allowed for user') % action
-            raise exception.HeatAccessDeniedError(msg)
-        except Exception:
+            raise exception.HeatAccessDeniedError("Action %s not allowed " %
+                                                  action + "for user")
+        except Exception as ex:
             # We expect policy.enforce to either pass or raise Forbidden
             # however, if anything else happens, we want to raise
             # HeatInternalFailureError, failure to do this results in
             # the user getting a big stacktrace spew as an API response
-            msg = _('Error authorizing action %s') % action
-            raise exception.HeatInternalFailureError(msg)
+            raise exception.HeatInternalFailureError("Error authorizing " +
+                                                     "action %s" % action)
 
     @staticmethod
     def _id_format(resp):
-        """Format the StackId field in the response as an ARN.
-
-        Also, process other IDs into the correct format.
+        """
+        Format the StackId field in the response as an ARN, and process other
+        IDs into the correct format.
         """
         if 'StackId' in resp:
             identity = identifier.HeatIdentifier(**resp['StackId'])
@@ -79,13 +80,14 @@ class StackController(object):
 
     @staticmethod
     def _extract_user_params(params):
-        """Extract a dictionary of user input parameters for the stack.
+        """
+        Extract a dictionary of user input parameters for the stack
 
         In the AWS API parameters, each user parameter appears as two key-value
-        pairs with keys of the form below::
+        pairs with keys of the form below:
 
-          Parameters.member.1.ParameterKey
-          Parameters.member.1.ParameterValue
+        Parameters.member.1.ParameterKey
+        Parameters.member.1.ParameterValue
         """
         return api_utils.extract_param_pairs(params,
                                              prefix='Parameters',
@@ -93,7 +95,8 @@ class StackController(object):
                                              valuename='ParameterValue')
 
     def _get_identity(self, con, stack_name):
-        """Generate a stack identifier from the given stack name or ARN.
+        """
+        Generate a stack identifier from the given stack name or ARN.
 
         In the case of a stack name, the identifier will be looked up in the
         engine over RPC.
@@ -101,44 +104,46 @@ class StackController(object):
         try:
             return dict(identifier.HeatIdentifier.from_arn(stack_name))
         except ValueError:
-            return self.rpc_client.identify_stack(con, stack_name)
+            return self.engine_rpcapi.identify_stack(con, stack_name)
 
     def list(self, req):
-        """Implements ListStacks API action.
-
-        Lists summary information for all stacks.
+        """
+        Implements ListStacks API action
+        Lists summary information for all stacks
         """
         self._enforce(req, 'ListStacks')
 
         def format_stack_summary(s):
-            """Reformat engine output into the AWS "StackSummary" format."""
+            """
+            Reformat engine output into the AWS "StackSummary" format
+            """
             # Map the engine-api format to the AWS StackSummary datatype
             keymap = {
-                rpc_api.STACK_CREATION_TIME: 'CreationTime',
-                rpc_api.STACK_UPDATED_TIME: 'LastUpdatedTime',
-                rpc_api.STACK_ID: 'StackId',
-                rpc_api.STACK_NAME: 'StackName',
-                rpc_api.STACK_STATUS_DATA: 'StackStatusReason',
-                rpc_api.STACK_TMPL_DESCRIPTION: 'TemplateDescription',
+                engine_api.STACK_CREATION_TIME: 'CreationTime',
+                engine_api.STACK_UPDATED_TIME: 'LastUpdatedTime',
+                engine_api.STACK_ID: 'StackId',
+                engine_api.STACK_NAME: 'StackName',
+                engine_api.STACK_STATUS_DATA: 'StackStatusReason',
+                engine_api.STACK_TMPL_DESCRIPTION: 'TemplateDescription',
             }
 
             result = api_utils.reformat_dict_keys(keymap, s)
 
-            action = s[rpc_api.STACK_ACTION]
-            status = s[rpc_api.STACK_STATUS]
+            action = s[engine_api.STACK_ACTION]
+            status = s[engine_api.STACK_STATUS]
             result['StackStatus'] = '_'.join((action, status))
 
-            # AWS docs indicate DeletionTime is omitted for current stacks
+            # AWS docs indicate DeletionTime is ommitted for current stacks
             # This is still TODO(unknown) in the engine, we don't keep data for
             # stacks after they are deleted
-            if rpc_api.STACK_DELETION_TIME in s:
-                result['DeletionTime'] = s[rpc_api.STACK_DELETION_TIME]
+            if engine_api.STACK_DELETION_TIME in s:
+                result['DeletionTime'] = s[engine_api.STACK_DELETION_TIME]
 
             return self._id_format(result)
 
         con = req.context
         try:
-            stack_list = self.rpc_client.list_stacks(con)
+            stack_list = self.engine_rpcapi.list_stacks(con)
         except Exception as ex:
             return exception.map_remote_error(ex)
 
@@ -147,27 +152,27 @@ class StackController(object):
         return api_utils.format_response('ListStacks', res)
 
     def describe(self, req):
-        """Implements DescribeStacks API action.
-
-        Gets detailed information for a stack (or all stacks).
+        """
+        Implements DescribeStacks API action
+        Gets detailed information for a stack (or all stacks)
         """
         self._enforce(req, 'DescribeStacks')
 
         def format_stack_outputs(o):
             keymap = {
-                rpc_api.OUTPUT_DESCRIPTION: 'Description',
-                rpc_api.OUTPUT_KEY: 'OutputKey',
-                rpc_api.OUTPUT_VALUE: 'OutputValue',
+                engine_api.OUTPUT_DESCRIPTION: 'Description',
+                engine_api.OUTPUT_KEY: 'OutputKey',
+                engine_api.OUTPUT_VALUE: 'OutputValue',
             }
 
             def replacecolon(d):
-                return dict(map(lambda k_v: (k_v[0].replace(':', '.'), k_v[1]),
+                return dict(map(lambda (k, v): (k.replace(':', '.'), v),
                                 d.items()))
 
             def transform(attrs):
-                """Recursively replace all `:` with `.` in dict keys.
-
-                After that they are not interpreted as xml namespaces.
+                """
+                Recursively replace all : with . in dict keys
+                so that they are not interpreted as xml namespaces.
                 """
                 new = replacecolon(attrs)
                 for key, value in new.items():
@@ -178,35 +183,35 @@ class StackController(object):
             return api_utils.reformat_dict_keys(keymap, transform(o))
 
         def format_stack(s):
-            """Reformat engine output into the AWS "StackSummary" format."""
+            """
+            Reformat engine output into the AWS "StackSummary" format
+            """
             keymap = {
-                rpc_api.STACK_CAPABILITIES: 'Capabilities',
-                rpc_api.STACK_CREATION_TIME: 'CreationTime',
-                rpc_api.STACK_DESCRIPTION: 'Description',
-                rpc_api.STACK_DISABLE_ROLLBACK: 'DisableRollback',
-                rpc_api.STACK_NOTIFICATION_TOPICS: 'NotificationARNs',
-                rpc_api.STACK_PARAMETERS: 'Parameters',
-                rpc_api.STACK_ID: 'StackId',
-                rpc_api.STACK_NAME: 'StackName',
-                rpc_api.STACK_STATUS_DATA: 'StackStatusReason',
-                rpc_api.STACK_TIMEOUT: 'TimeoutInMinutes',
+                engine_api.STACK_CAPABILITIES: 'Capabilities',
+                engine_api.STACK_CREATION_TIME: 'CreationTime',
+                engine_api.STACK_DESCRIPTION: 'Description',
+                engine_api.STACK_DISABLE_ROLLBACK: 'DisableRollback',
+                engine_api.STACK_UPDATED_TIME: 'LastUpdatedTime',
+                engine_api.STACK_NOTIFICATION_TOPICS: 'NotificationARNs',
+                engine_api.STACK_PARAMETERS: 'Parameters',
+                engine_api.STACK_ID: 'StackId',
+                engine_api.STACK_NAME: 'StackName',
+                engine_api.STACK_STATUS_DATA: 'StackStatusReason',
+                engine_api.STACK_TIMEOUT: 'TimeoutInMinutes',
             }
 
-            if s[rpc_api.STACK_UPDATED_TIME] is not None:
-                keymap[rpc_api.STACK_UPDATED_TIME] = 'LastUpdatedTime'
-
             result = api_utils.reformat_dict_keys(keymap, s)
 
-            action = s[rpc_api.STACK_ACTION]
-            status = s[rpc_api.STACK_STATUS]
+            action = s[engine_api.STACK_ACTION]
+            status = s[engine_api.STACK_STATUS]
             result['StackStatus'] = '_'.join((action, status))
 
             # Reformat outputs, these are handled separately as they are
             # only present in the engine output for a completely created
             # stack
             result['Outputs'] = []
-            if rpc_api.STACK_OUTPUTS in s:
-                for o in s[rpc_api.STACK_OUTPUTS]:
+            if engine_api.STACK_OUTPUTS in s:
+                for o in s[engine_api.STACK_OUTPUTS]:
                     result['Outputs'].append(format_stack_outputs(o))
 
             # Reformat Parameters dict-of-dict into AWS API format
@@ -228,7 +233,7 @@ class StackController(object):
             else:
                 identity = None
 
-            stack_list = self.rpc_client.show_stack(con, identity)
+            stack_list = self.engine_rpcapi.show_stack(con, identity)
 
         except Exception as ex:
             return exception.map_remote_error(ex)
@@ -238,17 +243,19 @@ class StackController(object):
         return api_utils.format_response('DescribeStacks', res)
 
     def _get_template(self, req):
-        """Get template file contents, either from local file or URL."""
+        """
+        Get template file contents, either from local file or URL
+        """
         if 'TemplateBody' in req.params:
-            LOG.debug('TemplateBody ...')
+            logger.debug('TemplateBody ...')
             return req.params['TemplateBody']
         elif 'TemplateUrl' in req.params:
             url = req.params['TemplateUrl']
-            LOG.debug('TemplateUrl %s' % url)
+            logger.debug('TemplateUrl %s' % url)
             try:
                 return urlfetch.get(url)
             except IOError as exc:
-                msg = _('Failed to fetch template: %s') % exc
+                msg = _('Failed to fetch template: %s') % str(exc)
                 raise exception.HeatInvalidParameterValueError(detail=msg)
 
         return None
@@ -268,19 +275,19 @@ class StackController(object):
         return self.create_or_update(req, self.UPDATE_STACK)
 
     def create_or_update(self, req, action=None):
-        """Implements CreateStack and UpdateStack API actions.
-
-        Create or update stack as defined in template file.
+        """
+        Implements CreateStack and UpdateStack API actions
+        Create or update stack as defined in template file
         """
         def extract_args(params):
-            """Extract request params and reformat them to match engine API.
-
-            FIXME: we currently only support a subset of
+            """
+            Extract request parameters/arguments and reformat them to match
+            the engine API.  FIXME: we currently only support a subset of
             the AWS defined parameters (both here and in the engine)
             """
             # TODO(shardy) : Capabilities, NotificationARNs
-            keymap = {'TimeoutInMinutes': rpc_api.PARAM_TIMEOUT,
-                      'DisableRollback': rpc_api.PARAM_DISABLE_ROLLBACK}
+            keymap = {'TimeoutInMinutes': engine_api.PARAM_TIMEOUT,
+                      'DisableRollback': engine_api.PARAM_DISABLE_ROLLBACK}
 
             if 'DisableRollback' in params and 'OnFailure' in params:
                 msg = _('DisableRollback and OnFailure '
@@ -296,9 +303,9 @@ class StackController(object):
             if 'OnFailure' in params:
                 value = params['OnFailure']
                 if value == 'DO_NOTHING':
-                    result[rpc_api.PARAM_DISABLE_ROLLBACK] = 'true'
+                    result[engine_api.PARAM_DISABLE_ROLLBACK] = 'true'
                 elif value in ('ROLLBACK', 'DELETE'):
-                    result[rpc_api.PARAM_DISABLE_ROLLBACK] = 'false'
+                    result[engine_api.PARAM_DISABLE_ROLLBACK] = 'false'
 
             return result
 
@@ -307,8 +314,8 @@ class StackController(object):
             # This should not happen, so return HeatInternalFailureError
             return exception.HeatInternalFailureError(detail=msg)
 
-        engine_action = {self.CREATE_STACK: self.rpc_client.create_stack,
-                         self.UPDATE_STACK: self.rpc_client.update_stack}
+        engine_action = {self.CREATE_STACK: self.engine_rpcapi.create_stack,
+                         self.UPDATE_STACK: self.engine_rpcapi.update_stack}
 
         con = req.context
 
@@ -358,41 +365,31 @@ class StackController(object):
 
         return api_utils.format_response(action, response)
 
-    def cancel_update(self, req):
-        action = 'CancelUpdateStack'
-        self._enforce(req, action)
-        con = req.context
-        stack_name = req.params['StackName']
-        stack_identity = self._get_identity(con, stack_name)
-        try:
-            self.rpc_client.stack_cancel_update(
-                con, stack_identity=stack_identity, cancel_with_rollback=True)
-        except Exception as ex:
-            return exception.map_remote_error(ex)
-
-        return api_utils.format_response(action, {})
-
     def get_template(self, req):
-        """Implements the GetTemplate API action.
-
-        Get the template body for an existing stack.
+        """
+        Implements the GetTemplate API action
+        Get the template body for an existing stack
         """
         self._enforce(req, 'GetTemplate')
 
         con = req.context
         try:
             identity = self._get_identity(con, req.params['StackName'])
-            templ = self.rpc_client.get_template(con, identity)
+            templ = self.engine_rpcapi.get_template(con, identity)
         except Exception as ex:
             return exception.map_remote_error(ex)
 
+        if templ is None:
+            msg = _('stack not not found')
+            return exception.HeatInvalidParameterValueError(detail=msg)
+
         return api_utils.format_response('GetTemplate',
                                          {'TemplateBody': templ})
 
     def estimate_template_cost(self, req):
-        """Implements the EstimateTemplateCost API action.
-
-        Get the estimated monthly cost of a template.
+        """
+        Implements the EstimateTemplateCost API action
+        Get the estimated monthly cost of a template
         """
         self._enforce(req, 'EstimateTemplateCost')
 
@@ -403,9 +400,9 @@ class StackController(object):
                                          )
 
     def validate_template(self, req):
-        """Implements the ValidateTemplate API action.
-
-        Validates the specified template.
+        """
+        Implements the ValidateTemplate API action
+        Validates the specified template
         """
         self._enforce(req, 'ValidateTemplate')
 
@@ -425,20 +422,22 @@ class StackController(object):
             msg = _("The Template must be a JSON or YAML document.")
             return exception.HeatInvalidParameterValueError(detail=msg)
 
-        LOG.info('validate_template')
+        logger.info('validate_template')
 
         def format_validate_parameter(key, value):
-            """Reformat engine output into AWS "ValidateTemplate" format."""
+            """
+            Reformat engine output into the AWS "ValidateTemplate" format
+            """
 
             return {
                 'ParameterKey': key,
-                'DefaultValue': value.get(rpc_api.PARAM_DEFAULT, ''),
-                'Description': value.get(rpc_api.PARAM_DESCRIPTION, ''),
-                'NoEcho': value.get(rpc_api.PARAM_NO_ECHO, 'false')
+                'DefaultValue': value.get(engine_api.PARAM_DEFAULT, ''),
+                'Description': value.get(engine_api.PARAM_DESCRIPTION, ''),
+                'NoEcho': value.get(engine_api.PARAM_NO_ECHO, 'false')
             }
 
         try:
-            res = self.rpc_client.validate_template(con, template)
+            res = self.engine_rpcapi.validate_template(con, template)
             if 'Error' in res:
                 return api_utils.format_response('ValidateTemplate',
                                                  res['Error'])
@@ -450,16 +449,16 @@ class StackController(object):
             return exception.map_remote_error(ex)
 
     def delete(self, req):
-        """Implements the DeleteStack API action.
-
-        Deletes the specified stack.
+        """
+        Implements the DeleteStack API action
+        Deletes the specified stack
         """
         self._enforce(req, 'DeleteStack')
 
         con = req.context
         try:
             identity = self._get_identity(con, req.params['StackName'])
-            res = self.rpc_client.delete_stack(con, identity, cast=False)
+            res = self.engine_rpcapi.delete_stack(con, identity, cast=False)
 
         except Exception as ex:
             return exception.map_remote_error(ex)
@@ -470,40 +469,42 @@ class StackController(object):
             return api_utils.format_response('DeleteStack', res['Error'])
 
     def events_list(self, req):
-        """Implements the DescribeStackEvents API action.
-
-        Returns events related to a specified stack (or all stacks).
+        """
+        Implements the DescribeStackEvents API action
+        Returns events related to a specified stack (or all stacks)
         """
         self._enforce(req, 'DescribeStackEvents')
 
         def format_stack_event(e):
-            """Reformat engine output into AWS "StackEvent" format."""
+            """
+            Reformat engine output into the AWS "StackEvent" format
+            """
             keymap = {
-                rpc_api.EVENT_ID: 'EventId',
-                rpc_api.EVENT_RES_NAME: 'LogicalResourceId',
-                rpc_api.EVENT_RES_PHYSICAL_ID: 'PhysicalResourceId',
-                rpc_api.EVENT_RES_PROPERTIES: 'ResourceProperties',
-                rpc_api.EVENT_RES_STATUS_DATA: 'ResourceStatusReason',
-                rpc_api.EVENT_RES_TYPE: 'ResourceType',
-                rpc_api.EVENT_STACK_ID: 'StackId',
-                rpc_api.EVENT_STACK_NAME: 'StackName',
-                rpc_api.EVENT_TIMESTAMP: 'Timestamp',
+                engine_api.EVENT_ID: 'EventId',
+                engine_api.EVENT_RES_NAME: 'LogicalResourceId',
+                engine_api.EVENT_RES_PHYSICAL_ID: 'PhysicalResourceId',
+                engine_api.EVENT_RES_PROPERTIES: 'ResourceProperties',
+                engine_api.EVENT_RES_STATUS_DATA: 'ResourceStatusReason',
+                engine_api.EVENT_RES_TYPE: 'ResourceType',
+                engine_api.EVENT_STACK_ID: 'StackId',
+                engine_api.EVENT_STACK_NAME: 'StackName',
+                engine_api.EVENT_TIMESTAMP: 'Timestamp',
             }
 
             result = api_utils.reformat_dict_keys(keymap, e)
-            action = e[rpc_api.EVENT_RES_ACTION]
-            status = e[rpc_api.EVENT_RES_STATUS]
+            action = e[engine_api.EVENT_RES_ACTION]
+            status = e[engine_api.EVENT_RES_STATUS]
             result['ResourceStatus'] = '_'.join((action, status))
-            result['ResourceProperties'] = jsonutils.dumps(result[
-                'ResourceProperties'])
+            result['ResourceProperties'] = json.dumps(result[
+                                                      'ResourceProperties'])
 
             return self._id_format(result)
 
         con = req.context
-        stack_name = req.params.get('StackName')
+        stack_name = req.params.get('StackName', None)
         try:
             identity = stack_name and self._get_identity(con, stack_name)
-            events = self.rpc_client.list_events(con, identity)
+            events = self.engine_rpcapi.list_events(con, identity)
         except Exception as ex:
             return exception.map_remote_error(ex)
 
@@ -514,29 +515,31 @@ class StackController(object):
 
     @staticmethod
     def _resource_status(res):
-        action = res[rpc_api.RES_ACTION]
-        status = res[rpc_api.RES_STATUS]
+        action = res[engine_api.RES_ACTION]
+        status = res[engine_api.RES_STATUS]
         return '_'.join((action, status))
 
     def describe_stack_resource(self, req):
-        """Implements the DescribeStackResource API action.
-
+        """
+        Implements the DescribeStackResource API action
         Return the details of the given resource belonging to the given stack.
         """
         self._enforce(req, 'DescribeStackResource')
 
         def format_resource_detail(r):
-            # Reformat engine output into the AWS "StackResourceDetail" format
+            """
+            Reformat engine output into the AWS "StackResourceDetail" format
+            """
             keymap = {
-                rpc_api.RES_DESCRIPTION: 'Description',
-                rpc_api.RES_UPDATED_TIME: 'LastUpdatedTimestamp',
-                rpc_api.RES_NAME: 'LogicalResourceId',
-                rpc_api.RES_METADATA: 'Metadata',
-                rpc_api.RES_PHYSICAL_ID: 'PhysicalResourceId',
-                rpc_api.RES_STATUS_DATA: 'ResourceStatusReason',
-                rpc_api.RES_TYPE: 'ResourceType',
-                rpc_api.RES_STACK_ID: 'StackId',
-                rpc_api.RES_STACK_NAME: 'StackName',
+                engine_api.RES_DESCRIPTION: 'Description',
+                engine_api.RES_UPDATED_TIME: 'LastUpdatedTimestamp',
+                engine_api.RES_NAME: 'LogicalResourceId',
+                engine_api.RES_METADATA: 'Metadata',
+                engine_api.RES_PHYSICAL_ID: 'PhysicalResourceId',
+                engine_api.RES_STATUS_DATA: 'ResourceStatusReason',
+                engine_api.RES_TYPE: 'ResourceType',
+                engine_api.RES_STACK_ID: 'StackId',
+                engine_api.RES_STACK_NAME: 'StackName',
             }
 
             result = api_utils.reformat_dict_keys(keymap, r)
@@ -549,7 +552,7 @@ class StackController(object):
 
         try:
             identity = self._get_identity(con, req.params['StackName'])
-            resource_details = self.rpc_client.describe_stack_resource(
+            resource_details = self.engine_rpcapi.describe_stack_resource(
                 con,
                 stack_identity=identity,
                 resource_name=req.params.get('LogicalResourceId'))
@@ -563,14 +566,13 @@ class StackController(object):
                                          {'StackResourceDetail': result})
 
     def describe_stack_resources(self, req):
-        """Implements the DescribeStackResources API action.
-
+        """
+        Implements the DescribeStackResources API action
         Return details of resources specified by the parameters.
 
-        `StackName`: returns all resources belonging to the stack.
-
+        `StackName`: returns all resources belonging to the stack
         `PhysicalResourceId`: returns all resources belonging to the stack this
-        resource is associated with.
+                              resource is associated with.
 
         Only one of the parameters may be specified.
 
@@ -582,16 +584,18 @@ class StackController(object):
         self._enforce(req, 'DescribeStackResources')
 
         def format_stack_resource(r):
-            """Reformat engine output into AWS "StackResource" format."""
+            """
+            Reformat engine output into the AWS "StackResource" format
+            """
             keymap = {
-                rpc_api.RES_DESCRIPTION: 'Description',
-                rpc_api.RES_NAME: 'LogicalResourceId',
-                rpc_api.RES_PHYSICAL_ID: 'PhysicalResourceId',
-                rpc_api.RES_STATUS_DATA: 'ResourceStatusReason',
-                rpc_api.RES_TYPE: 'ResourceType',
-                rpc_api.RES_STACK_ID: 'StackId',
-                rpc_api.RES_STACK_NAME: 'StackName',
-                rpc_api.RES_UPDATED_TIME: 'Timestamp',
+                engine_api.RES_DESCRIPTION: 'Description',
+                engine_api.RES_NAME: 'LogicalResourceId',
+                engine_api.RES_PHYSICAL_ID: 'PhysicalResourceId',
+                engine_api.RES_STATUS_DATA: 'ResourceStatusReason',
+                engine_api.RES_TYPE: 'ResourceType',
+                engine_api.RES_STACK_ID: 'StackId',
+                engine_api.RES_STACK_NAME: 'StackName',
+                engine_api.RES_UPDATED_TIME: 'Timestamp',
             }
 
             result = api_utils.reformat_dict_keys(keymap, r)
@@ -611,10 +615,10 @@ class StackController(object):
             if stack_name is not None:
                 identity = self._get_identity(con, stack_name)
             else:
-                identity = self.rpc_client.find_physical_resource(
+                identity = self.engine_rpcapi.find_physical_resource(
                     con,
                     physical_resource_id=physical_resource_id)
-            resources = self.rpc_client.describe_stack_resources(
+            resources = self.engine_rpcapi.describe_stack_resources(
                 con,
                 stack_identity=identity,
                 resource_name=req.params.get('LogicalResourceId'))
@@ -628,20 +632,22 @@ class StackController(object):
                                          {'StackResources': result})
 
     def list_stack_resources(self, req):
-        """Implements the ListStackResources API action.
-
+        """
+        Implements the ListStackResources API action
         Return summary of the resources belonging to the specified stack.
         """
         self._enforce(req, 'ListStackResources')
 
         def format_resource_summary(r):
-            """Reformat engine output to AWS "StackResourceSummary" format."""
+            """
+            Reformat engine output into the AWS "StackResourceSummary" format
+            """
             keymap = {
-                rpc_api.RES_UPDATED_TIME: 'LastUpdatedTimestamp',
-                rpc_api.RES_NAME: 'LogicalResourceId',
-                rpc_api.RES_PHYSICAL_ID: 'PhysicalResourceId',
-                rpc_api.RES_STATUS_DATA: 'ResourceStatusReason',
-                rpc_api.RES_TYPE: 'ResourceType',
+                engine_api.RES_UPDATED_TIME: 'LastUpdatedTimestamp',
+                engine_api.RES_NAME: 'LogicalResourceId',
+                engine_api.RES_PHYSICAL_ID: 'PhysicalResourceId',
+                engine_api.RES_STATUS_DATA: 'ResourceStatusReason',
+                engine_api.RES_TYPE: 'ResourceType',
             }
 
             result = api_utils.reformat_dict_keys(keymap, r)
@@ -654,7 +660,7 @@ class StackController(object):
 
         try:
             identity = self._get_identity(con, req.params['StackName'])
-            resources = self.rpc_client.list_stack_resources(
+            resources = self.engine_rpcapi.list_stack_resources(
                 con,
                 stack_identity=identity)
         except Exception as ex:
@@ -667,6 +673,8 @@ class StackController(object):
 
 
 def create_resource(options):
-    """Stacks resource factory method."""
+    """
+    Stacks resource factory method.
+    """
     deserializer = wsgi.JSONRequestDeserializer()
     return wsgi.Resource(StackController(options), deserializer)
