@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -11,113 +13,120 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-"""Endpoint for heat AWS-compatible CloudWatch API."""
-
-from oslo_log import log as logging
-import oslo_messaging as messaging
-import six
-
+"""
+endpoint for heat AWS-compatible CloudWatch API
+"""
 from heat.api.aws import exception
 from heat.api.aws import utils as api_utils
-from heat.common import exception as heat_exception
-from heat.common.i18n import _
-from heat.common import policy
 from heat.common import wsgi
-from heat.rpc import api as rpc_api
+from heat.common import policy
+from heat.common import exception as heat_exception
 from heat.rpc import client as rpc_client
+from heat.rpc import api as engine_api
 
-LOG = logging.getLogger(__name__)
+import heat.openstack.common.rpc.common as rpc_common
+from heat.openstack.common import log as logging
+
+logger = logging.getLogger(__name__)
 
 
 class WatchController(object):
 
-    """WSGI controller for CloudWatch resource in heat API.
-
-    Implements the API actions.
+    """
+    WSGI controller for CloudWatch resource in heat API
+    Implements the API actions
     """
 
     def __init__(self, options):
         self.options = options
-        self.rpc_client = rpc_client.EngineClient()
+        self.engine_rpcapi = rpc_client.EngineClient()
         self.policy = policy.Enforcer(scope='cloudwatch')
 
     def _enforce(self, req, action):
         """Authorize an action against the policy.json."""
         try:
-            self.policy.enforce(req.context, action)
+            self.policy.enforce(req.context, action, {})
         except heat_exception.Forbidden:
-            msg = _("Action %s not allowed for user") % action
-            raise exception.HeatAccessDeniedError(msg)
-        except Exception:
+            raise exception.HeatAccessDeniedError("Action %s not allowed " %
+                                                  action + "for user")
+        except Exception as ex:
             # We expect policy.enforce to either pass or raise Forbidden
             # however, if anything else happens, we want to raise
             # HeatInternalFailureError, failure to do this results in
             # the user getting a big stacktrace spew as an API response
-            msg = _("Error authorizing action %s") % action
-            raise exception.HeatInternalFailureError(msg)
+            raise exception.HeatInternalFailureError("Error authorizing " +
+                                                     "action %s" % action)
 
     @staticmethod
     def _reformat_dimensions(dims):
-        """Reformat dimensions list into AWS API format.
-
-        :param dims: a list of dicts.
-        """
+        '''
+        Reformat dimensions list into AWS API format
+        Parameter dims is a list of dicts
+        '''
         newdims = []
         for count, d in enumerate(dims, 1):
-            for key, value in d.items():
-                newdims.append({'Name': key, 'Value': value})
+            for key in d.keys():
+                newdims.append({'Name': key, 'Value': d[key]})
         return newdims
 
     def delete_alarms(self, req):
-        """Implements DeleteAlarms API action."""
+        """
+        Implements DeleteAlarms API action
+        """
         self._enforce(req, 'DeleteAlarms')
         return exception.HeatAPINotImplementedError()
 
     def describe_alarm_history(self, req):
-        """Implements DescribeAlarmHistory API action."""
+        """
+        Implements DescribeAlarmHistory API action
+        """
         self._enforce(req, 'DescribeAlarmHistory')
         return exception.HeatAPINotImplementedError()
 
     def describe_alarms(self, req):
-        """Implements DescribeAlarms API action."""
+        """
+        Implements DescribeAlarms API action
+        """
         self._enforce(req, 'DescribeAlarms')
 
         def format_metric_alarm(a):
-            """Reformat engine output into the AWS "MetricAlarm" format."""
+            """
+            Reformat engine output into the AWS "MetricAlarm" format
+            """
             keymap = {
-                rpc_api.WATCH_ACTIONS_ENABLED: 'ActionsEnabled',
-                rpc_api.WATCH_ALARM_ACTIONS: 'AlarmActions',
-                rpc_api.WATCH_TOPIC: 'AlarmArn',
-                rpc_api.WATCH_UPDATED_TIME:
+                engine_api.WATCH_ACTIONS_ENABLED: 'ActionsEnabled',
+                engine_api.WATCH_ALARM_ACTIONS: 'AlarmActions',
+                engine_api.WATCH_TOPIC: 'AlarmArn',
+                engine_api.WATCH_UPDATED_TIME:
                 'AlarmConfigurationUpdatedTimestamp',
-                rpc_api.WATCH_DESCRIPTION: 'AlarmDescription',
-                rpc_api.WATCH_NAME: 'AlarmName',
-                rpc_api.WATCH_COMPARISON: 'ComparisonOperator',
-                rpc_api.WATCH_DIMENSIONS: 'Dimensions',
-                rpc_api.WATCH_PERIODS: 'EvaluationPeriods',
-                rpc_api.WATCH_INSUFFICIENT_ACTIONS:
+                engine_api.WATCH_DESCRIPTION: 'AlarmDescription',
+                engine_api.WATCH_NAME: 'AlarmName',
+                engine_api.WATCH_COMPARISON: 'ComparisonOperator',
+                engine_api.WATCH_DIMENSIONS: 'Dimensions',
+                engine_api.WATCH_PERIODS: 'EvaluationPeriods',
+                engine_api.WATCH_INSUFFICIENT_ACTIONS:
                 'InsufficientDataActions',
-                rpc_api.WATCH_METRIC_NAME: 'MetricName',
-                rpc_api.WATCH_NAMESPACE: 'Namespace',
-                rpc_api.WATCH_OK_ACTIONS: 'OKActions',
-                rpc_api.WATCH_PERIOD: 'Period',
-                rpc_api.WATCH_STATE_REASON: 'StateReason',
-                rpc_api.WATCH_STATE_REASON_DATA: 'StateReasonData',
-                rpc_api.WATCH_STATE_UPDATED_TIME: 'StateUpdatedTimestamp',
-                rpc_api.WATCH_STATE_VALUE: 'StateValue',
-                rpc_api.WATCH_STATISTIC: 'Statistic',
-                rpc_api.WATCH_THRESHOLD: 'Threshold',
-                rpc_api.WATCH_UNIT: 'Unit',
-            }
+                engine_api.WATCH_METRIC_NAME: 'MetricName',
+                engine_api.WATCH_NAMESPACE: 'Namespace',
+                engine_api.WATCH_OK_ACTIONS: 'OKActions',
+                engine_api.WATCH_PERIOD: 'Period',
+                engine_api.WATCH_STATE_REASON: 'StateReason',
+                engine_api.WATCH_STATE_REASON_DATA: 'StateReasonData',
+                engine_api.WATCH_STATE_UPDATED_TIME: 'StateUpdatedTimestamp',
+                engine_api.WATCH_STATE_VALUE: 'StateValue',
+                engine_api.WATCH_STATISTIC: 'Statistic',
+                engine_api.WATCH_THRESHOLD: 'Threshold',
+                engine_api.WATCH_UNIT: 'Unit'}
 
             # AWS doesn't return StackId in the main MetricAlarm
             # structure, so we add StackId as a dimension to all responses
-            a[rpc_api.WATCH_DIMENSIONS].append({'StackId':
-                                                a[rpc_api.WATCH_STACK_ID]})
+            a[engine_api.WATCH_DIMENSIONS].append({'StackId':
+                                                  a[engine_api.WATCH_STACK_ID]
+                                                   })
 
             # Reformat dimensions list into AWS API format
-            a[rpc_api.WATCH_DIMENSIONS] = self._reformat_dimensions(
-                a[rpc_api.WATCH_DIMENSIONS])
+            a[engine_api.WATCH_DIMENSIONS] = self._reformat_dimensions(
+                a[engine_api.WATCH_DIMENSIONS])
 
             return api_utils.reformat_dict_keys(keymap, a)
 
@@ -129,8 +138,8 @@ class WatchController(object):
             name = None
 
         try:
-            watch_list = self.rpc_client.show_watch(con, watch_name=name)
-        except messaging.RemoteError as ex:
+            watch_list = self.engine_rpcapi.show_watch(con, watch_name=name)
+        except rpc_common.RemoteError as ex:
             return exception.map_remote_error(ex)
 
         res = {'MetricAlarms': [format_metric_alarm(a)
@@ -140,54 +149,61 @@ class WatchController(object):
         return result
 
     def describe_alarms_for_metric(self, req):
-        """Implements DescribeAlarmsForMetric API action."""
+        """
+        Implements DescribeAlarmsForMetric API action
+        """
         self._enforce(req, 'DescribeAlarmsForMetric')
         return exception.HeatAPINotImplementedError()
 
     def disable_alarm_actions(self, req):
-        """Implements DisableAlarmActions API action."""
+        """
+        Implements DisableAlarmActions API action
+        """
         self._enforce(req, 'DisableAlarmActions')
         return exception.HeatAPINotImplementedError()
 
     def enable_alarm_actions(self, req):
-        """Implements EnableAlarmActions API action."""
+        """
+        Implements EnableAlarmActions API action
+        """
         self._enforce(req, 'EnableAlarmActions')
         return exception.HeatAPINotImplementedError()
 
     def get_metric_statistics(self, req):
-        """Implements GetMetricStatistics API action."""
+        """
+        Implements GetMetricStatistics API action
+        """
         self._enforce(req, 'GetMetricStatistics')
         return exception.HeatAPINotImplementedError()
 
     def list_metrics(self, req):
-        """Implements ListMetrics API action.
-
+        """
+        Implements ListMetrics API action
         Lists metric datapoints associated with a particular alarm,
-        or all alarms if none specified.
+        or all alarms if none specified
         """
         self._enforce(req, 'ListMetrics')
 
-        def format_metric_data(d, fil=None):
-            """Reformat engine output into the AWS "Metric" format.
-
+        def format_metric_data(d, fil={}):
+            """
+            Reformat engine output into the AWS "Metric" format
             Takes an optional filter dict, which is traversed
             so a metric dict is only returned if all keys match
-            the filter dict.
+            the filter dict
             """
-            fil = fil or {}
             dimensions = [
-                {'AlarmName': d[rpc_api.WATCH_DATA_ALARM]},
-                {'Timestamp': d[rpc_api.WATCH_DATA_TIME]}
+                {'AlarmName': d[engine_api.WATCH_DATA_ALARM]},
+                {'Timestamp': d[engine_api.WATCH_DATA_TIME]}
             ]
-            for key in d[rpc_api.WATCH_DATA]:
-                dimensions.append({key: d[rpc_api.WATCH_DATA][key]})
+            for key in d[engine_api.WATCH_DATA]:
+                dimensions.append({key: d[engine_api.WATCH_DATA][key]})
 
             newdims = self._reformat_dimensions(dimensions)
 
             result = {
-                'MetricName': d[rpc_api.WATCH_DATA_METRIC],
+                'MetricName': d[engine_api.WATCH_DATA_METRIC],
                 'Dimensions': newdims,
-                'Namespace': d[rpc_api.WATCH_DATA_NAMESPACE],
+                'Namespace': d[engine_api.WATCH_DATA_NAMESPACE],
             }
 
             for f in fil:
@@ -197,25 +213,25 @@ class WatchController(object):
                         # Filter criteria not met, return None
                         return
                 except KeyError:
-                    LOG.warning("Invalid filter key %s, ignoring", f)
+                    logger.warning("Invalid filter key %s, ignoring" % f)
 
             return result
 
         con = req.context
         parms = dict(req.params)
         # FIXME : Don't yet handle filtering by Dimensions
-        filter_result = dict((k, v) for (k, v) in six.iteritems(parms) if k in
+        filter_result = dict((k, v) for (k, v) in parms.iteritems() if k in
                              ("MetricName", "Namespace"))
-        LOG.debug("filter parameters : %s" % filter_result)
+        logger.debug("filter parameters : %s" % filter_result)
 
         try:
             # Engine does not currently support query by namespace/metric
             # so we pass None/None and do any filtering locally
             null_kwargs = {'metric_namespace': None,
                            'metric_name': None}
-            watch_data = self.rpc_client.show_watch_metric(con,
-                                                           **null_kwargs)
-        except messaging.RemoteError as ex:
+            watch_data = self.engine_rpcapi.show_watch_metric(con,
+                                                              **null_kwargs)
+        except rpc_common.RemoteError as ex:
             return exception.map_remote_error(ex)
 
         res = {'Metrics': []}
@@ -228,12 +244,16 @@ class WatchController(object):
         return result
 
     def put_metric_alarm(self, req):
-        """Implements PutMetricAlarm API action."""
+        """
+        Implements PutMetricAlarm API action
+        """
         self._enforce(req, 'PutMetricAlarm')
         return exception.HeatAPINotImplementedError()
 
     def put_metric_data(self, req):
-        """Implements PutMetricData API action."""
+        """
+        Implements PutMetricData API action
+        """
         self._enforce(req, 'PutMetricData')
 
         con = req.context
@@ -248,8 +268,8 @@ class WatchController(object):
         # need to process (each dict) for dimensions
         metric_data = api_utils.extract_param_list(parms, prefix='MetricData')
         if not len(metric_data):
-            LOG.error("Request does not contain required MetricData")
-            return exception.HeatMissingParameterError(_("MetricData list"))
+            logger.error("Request does not contain required MetricData")
+            return exception.HeatMissingParameterError("MetricData list")
 
         watch_name = None
         dimensions = []
@@ -273,21 +293,23 @@ class WatchController(object):
                     'Dimensions': dimensions}}
 
         try:
-            self.rpc_client.create_watch_data(con, watch_name, data)
-        except messaging.RemoteError as ex:
+            self.engine_rpcapi.create_watch_data(con, watch_name, data)
+        except rpc_common.RemoteError as ex:
             return exception.map_remote_error(ex)
 
         result = {'ResponseMetadata': None}
         return api_utils.format_response("PutMetricData", result)
 
     def set_alarm_state(self, req):
-        """Implements SetAlarmState API action."""
+        """
+        Implements SetAlarmState API action
+        """
         self._enforce(req, 'SetAlarmState')
 
         # Map from AWS state names to those used in the engine
-        state_map = {'OK': rpc_api.WATCH_STATE_OK,
-                     'ALARM': rpc_api.WATCH_STATE_ALARM,
-                     'INSUFFICIENT_DATA': rpc_api.WATCH_STATE_NODATA}
+        state_map = {'OK': engine_api.WATCH_STATE_OK,
+                     'ALARM': engine_api.WATCH_STATE_ALARM,
+                     'INSUFFICIENT_DATA': engine_api.WATCH_STATE_NODATA}
 
         con = req.context
         parms = dict(req.params)
@@ -297,25 +319,33 @@ class WatchController(object):
         state = api_utils.get_param_value(parms, 'StateValue')
 
         if state not in state_map:
-            msg = _('Invalid state %(state)s, '
-                    'expecting one of %(expect)s') % {
-                        'state': state,
-                        'expect': list(state_map.keys())}
-            LOG.error(msg)
-            return exception.HeatInvalidParameterValueError(msg)
+            logger.error("Invalid state %s, expecting one of %s" %
+                         (state, state_map.keys()))
+            return exception.HeatInvalidParameterValueError("Invalid state %s"
+                                                            % state)
 
-        LOG.debug("setting %(name)s to %(state)s" % {
-                  'name': name, 'state': state_map[state]})
+        # Check for optional parameters
+        # FIXME : We don't actually do anything with these in the engine yet..
+        state_reason = None
+        state_reason_data = None
+        if 'StateReason' in parms:
+            state_reason = parms['StateReason']
+        if 'StateReasonData' in parms:
+            state_reason_data = parms['StateReasonData']
+
+        logger.debug("setting %s to %s" % (name, state_map[state]))
         try:
-            self.rpc_client.set_watch_state(con, watch_name=name,
-                                            state=state_map[state])
-        except messaging.RemoteError as ex:
+            self.engine_rpcapi.set_watch_state(con, watch_name=name,
+                                               state=state_map[state])
+        except rpc_common.RemoteError as ex:
             return exception.map_remote_error(ex)
 
         return api_utils.format_response("SetAlarmState", "")
 
 
 def create_resource(options):
-    """Watch resource factory method."""
+    """
+    Watch resource factory method.
+    """
     deserializer = wsgi.JSONRequestDeserializer()
     return wsgi.Resource(WatchController(options), deserializer)
