@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -11,23 +13,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from debtcollector import removals
-from heat.api.middleware import fault
-from heat.api.middleware import ssl
-from heat.api.middleware import version_negotiation as vn
+from heat.api.middleware.version_negotiation import VersionNegotiationFilter
+from heat.api.middleware.fault import FaultWrapper
 from heat.api.openstack import versions
 
 
 def version_negotiation_filter(app, conf, **local_conf):
-    return vn.VersionNegotiationFilter(versions.Controller, app,
-                                       conf, **local_conf)
+    return VersionNegotiationFilter(versions.Controller, app,
+                                    conf, **local_conf)
 
 
 def faultwrap_filter(app, conf, **local_conf):
-    return fault.FaultWrapper(app)
-
-
-@removals.remove(message='Use oslo_middleware.http_proxy_to_wsgi instead.',
-                 version='6.0.0', removal_version='8.0.0')
-def sslmiddleware_filter(app, conf, **local_conf):
-    return ssl.SSLMiddleware(app)
+    return FaultWrapper(app)
