@@ -14,28 +14,32 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import gettext
+from __future__ import print_function
 
 import sys
 
-gettext.install('heat', unicode=1)
+from heat.openstack.common import gettextutils
+
+gettextutils.install('heat')
 
 from oslo.config import cfg
 from heat.openstack.common import log as logging
-import heat.db
+from heat.db import api
 from heat.db import migration
 
-
 LOG = logging.getLogger(__name__)
 
 
 if __name__ == '__main__':
+    print('*******************************************', file=sys.stderr)
+    print('Deprecated: use heat-manage db_sync instead', file=sys.stderr)
+    print('*******************************************', file=sys.stderr)
     cfg.CONF(project='heat', prog='heat-engine')
 
-    heat.db.configure()
+    api.configure()
 
     try:
         migration.db_sync()
     except Exception as exc:
-        print >>sys.stderr, str(exc)
+        print(str(exc), file=sys.stderr)
         sys.exit(1)
