@@ -12,17 +12,20 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+import os
+
+from oslo.config import cfg
 
-from heat.engine import clients
 from heat.common import exception
 from heat.common import template_format
 from heat.engine import stack_resource
+from heat.engine.resources import nova_utils
 
 from heat.openstack.common import log as logging
 
 logger = logging.getLogger(__name__)
 
-lb_template = r'''
+lb_template_default = r'''
 {
   "AWSTemplateFormatVersion": "2010-09-09",
   "Description": "Built in HAProxy server",
@@ -69,8 +72,7 @@ lb_template = r'''
             },
             "services": {
               "systemd": {
-                "crond"     : { "enabled" : "true", "ensureRunning" : "true" },
-                "haproxy"   : { "enabled": "true", "ensureRunning": "true" }
+                "crond"     : { "enabled" : "true", "ensureRunning" : "true" }
               }
             },
             "files": {
@@ -110,7 +112,7 @@ lb_template = r'''
                   "[reload]\n",
                   "triggers=post.update\n",
                   "path=Resources.LB_instance.Metadata\n",
-                  "action=systemctl reload haproxy.service\n",
+                  "action=systemctl reload-or-restart haproxy.service\n",
                   "runas=root\n"
                 ]]},
                 "mode"    : "000400",
@@ -191,71 +193,118 @@ lb_template = r'''
 '''
 
 
-#
-# TODO(asalkeld) the above inline template _could_ be placed in an external
-# file at the moment this is because we will probably need to implement a
-# LoadBalancer based on keepalived as well (for for ssl support).
-#
+# Allow user to provide alternative nested stack template to the above
+loadbalancer_opts = [
+    cfg.StrOpt('loadbalancer_template',
+               default=None,
+               help='Custom template for the built-in '
+                    'loadbalancer nested stack')]
+cfg.CONF.register_opts(loadbalancer_opts)
+
+
 class LoadBalancer(stack_resource.StackResource):
 
     listeners_schema = {
-        'InstancePort': {'Type': 'Number',
-                         'Required': True},
-        'LoadBalancerPort': {'Type': 'Number',
-                             'Required': True},
-        'Protocol': {'Type': 'String',
-                     'Required': True,
-                     'AllowedValues': ['TCP', 'HTTP']},
-        'SSLCertificateId': {'Type': 'String',
-                             'Implemented': False},
-        'PolicyNames': {'Type': 'List',
-                        'Implemented': False}
+        'InstancePort': {
+            'Type': 'Number',
+            'Required': True,
+            'Description': _('TCP port on which the instance server is'
+                             ' listening.')},
+        'LoadBalancerPort': {
+            'Type': 'Number',
+            'Required': True,
+            'Description': _('The external load balancer port number.')},
+        'Protocol': {
+            'Type': 'String',
+            'Required': True,
+            'AllowedValues': ['TCP', 'HTTP'],
+            'Description': _('The load balancer transport protocol to use.')},
+        'SSLCertificateId': {
+            'Type': 'String',
+            'Implemented': False,
+            'Description': _('Not Implemented.')},
+        'PolicyNames': {
+            'Type': 'List',
+            'Implemented': False,
+            'Description': _('Not Implemented.')}
     }
     healthcheck_schema = {
-        'HealthyThreshold': {'Type': 'Number',
-                             'Required': True},
-        'Interval': {'Type': 'Number',
-                     'Required': True},
-        'Target': {'Type': 'String',
-                   'Required': True},
-        'Timeout': {'Type': 'Number',
-                    'Required': True},
-        'UnhealthyThreshold': {'Type': 'Number',
-                               'Required': True},
+        'HealthyThreshold': {
+            'Type': 'Number',
+            'Required': True,
+            'Description': _('The number of consecutive health probe successes'
+                             ' required before moving the instance to the'
+                             ' healthy state.')},
+        'Interval': {
+            'Type': 'Number',
+            'Required': True,
+            'Description': _('The approximate interval, in seconds, between'
+                             ' health checks of an individual instance.')},
+        'Target': {
+            'Type': 'String',
+            'Required': True,
+            'Description': _('The port being checked.')},
+        'Timeout': {
+            'Type': 'Number',
+            'Required': True,
+            'Description': _('Health probe timeout, in seconds.')},
+        'UnhealthyThreshold': {
+            'Type': 'Number',
+            'Required': True,
+            'Description': _('The number of consecutive health probe failures'
+                             ' required before moving the instance to the'
+                             ' unhealthy state')},
     }
 
     properties_schema = {
-        'AvailabilityZones': {'Type': 'List',
-                              'Required': True},
-        'HealthCheck': {'Type': 'Map',
-                        'Schema': healthcheck_schema},
-        'Instances': {'Type': 'List'},
-        'Listeners': {'Type': 'List',
-                      'Schema': {'Type': 'Map',
-                                 'Schema': listeners_schema}},
-        'AppCookieStickinessPolicy': {'Type': 'String',
-                                      'Implemented': False},
-        'LBCookieStickinessPolicy': {'Type': 'String',
-                                     'Implemented': False},
-        'SecurityGroups': {'Type': 'String',
-                           'Implemented': False},
-        'Subnets': {'Type': 'List',
-                    'Implemented': False}
+        'AvailabilityZones': {
+            'Type': 'List',
+            'Required': True,
+            'Description': _('The Availability Zones in which to create the'
+                             ' load balancer.')},
+        'HealthCheck': {
+            'Type': 'Map',
+            'Schema': healthcheck_schema,
+            'Description': _('An application health check for the'
+                             ' instances.')},
+        'Instances': {
+            'Type': 'List',
+            'Description': _('The list of instance IDs load balanced.')},
+        'Listeners': {
+            'Type': 'List', 'Required': True,
+            'Schema': {'Type': 'Map', 'Schema': listeners_schema},
+            'Description': _('One or more listeners for this load balancer.')},
+        'AppCookieStickinessPolicy': {
+            'Type': 'String',
+            'Implemented': False,
+            'Description': _('Not Implemented.')},
+        'LBCookieStickinessPolicy': {
+            'Type': 'String',
+            'Implemented': False,
+            'Description': _('Not Implemented.')},
+        'SecurityGroups': {
+            'Type': 'String',
+            'Implemented': False,
+            'Description': _('Not Implemented.')},
+        'Subnets': {
+            'Type': 'List',
+            'Implemented': False,
+            'Description': _('Not Implemented.')}
     }
-
-    def _instance_to_ipaddress(self, inst):
-        '''
-        Return the server's IP address, fetching it from Nova
-        '''
-        try:
-            server = self.nova().servers.get(inst)
-        except clients.novaclient.exceptions.NotFound as ex:
-            logger.warn('Instance (%s) not found: %s' % (inst, str(ex)))
-        else:
-            for n in server.networks:
-                return server.networks[n][0]
-
-        return '0.0.0.0'
+    attributes_schema = {
+        "CanonicalHostedZoneName": ("The name of the hosted zone that is "
+                                    "associated with the LoadBalancer."),
+        "CanonicalHostedZoneNameID": ("The ID of the hosted zone name that is "
+                                      "associated with the LoadBalancer."),
+        "DNSName": "The DNS name for the LoadBalancer.",
+        "SourceSecurityGroup.GroupName": ("The security group that you can use"
+                                          " as part of your inbound rules for "
+                                          "your LoadBalancer's back-end "
+                                          "instances."),
+        "SourceSecurityGroup.OwnerAlias": "Owner of the source security group."
+    }
+    update_allowed_keys = ('Properties',)
+    update_allowed_properties = ('Instances',)
 
     def _haproxy_config(self, templ, instances):
         # initial simplifications:
@@ -309,8 +358,9 @@ class LoadBalancer(stack_resource.StackResource):
 
         servers = []
         n = 1
+        client = self.nova()
         for i in instances:
-            ip = self._instance_to_ipaddress(i)
+            ip = nova_utils.server_to_ipaddress(client, i) or '0.0.0.0'
             logger.debug('haproxy server:%s' % ip)
             servers.append('%sserver server%d %s:%s %s' % (spaces, n,
                                                            ip, inst_port,
@@ -319,8 +369,18 @@ class LoadBalancer(stack_resource.StackResource):
 
         return '%s%s%s%s\n' % (gl, frontend, backend, '\n'.join(servers))
 
+    def get_parsed_template(self):
+        if cfg.CONF.loadbalancer_template:
+            with open(cfg.CONF.loadbalancer_template) as templ_fd:
+                logger.info(_('Using custom loadbalancer template %s')
+                            % cfg.CONF.loadbalancer_template)
+                contents = templ_fd.read()
+        else:
+            contents = lb_template_default
+        return template_format.parse(contents)
+
     def handle_create(self):
-        templ = template_format.parse(lb_template)
+        templ = self.get_parsed_template()
 
         if self.properties['Instances']:
             md = templ['Resources']['LB_instance']['Metadata']
@@ -328,16 +388,35 @@ class LoadBalancer(stack_resource.StackResource):
             cfg = self._haproxy_config(templ, self.properties['Instances'])
             files['/etc/haproxy/haproxy.cfg']['content'] = cfg
 
-        # total hack - probably need an admin key here.
-        param = self.stack.resolve_static_data({'KeyName': {'Ref': 'KeyName'}})
+        # If the owning stack defines KeyName, we use that key for the nested
+        # template, otherwise use no key
+        try:
+            param = {'KeyName': self.stack.parameters['KeyName']}
+        except KeyError:
+            del templ['Resources']['LB_instance']['Properties']['KeyName']
+            del templ['Parameters']['KeyName']
+            param = {}
 
-        self.create_with_template(templ, param)
+        return self.create_with_template(templ, param)
 
-    def handle_update(self, json_snippet):
-        return self.UPDATE_REPLACE
+    def handle_update(self, json_snippet, tmpl_diff, prop_diff):
+        '''
+        re-generate the Metadata
+        save it to the db.
+        rely on the cfn-hup to reconfigure HAProxy
+        '''
+        if 'Instances' in prop_diff:
+            templ = self.get_parsed_template()
+            cfg = self._haproxy_config(templ, prop_diff['Instances'])
+
+            md = self.nested()['LB_instance'].metadata
+            files = md['AWS::CloudFormation::Init']['config']['files']
+            files['/etc/haproxy/haproxy.cfg']['content'] = cfg
+
+            self.nested()['LB_instance'].metadata = md
 
     def handle_delete(self):
-        self.delete_nested()
+        return self.delete_nested()
 
     def validate(self):
         '''
@@ -347,47 +426,29 @@ class LoadBalancer(stack_resource.StackResource):
         if res:
             return res
 
+        if cfg.CONF.loadbalancer_template and \
+                not os.access(cfg.CONF.loadbalancer_template, os.R_OK):
+            msg = _('Custom LoadBalancer template can not be found')
+            raise exception.StackValidationFailed(message=msg)
+
         health_chk = self.properties['HealthCheck']
         if health_chk:
             if float(health_chk['Interval']) < float(health_chk['Timeout']):
                 return {'Error':
                         'Interval must be larger than Timeout'}
 
-    def reload(self, inst_list):
-        '''
-        re-generate the Metadata
-        save it to the db.
-        rely on the cfn-hup to reconfigure HAProxy
-        '''
-        templ = template_format.parse(lb_template)
-        cfg = self._haproxy_config(templ, inst_list)
-
-        md = self.nested()['LB_instance'].metadata
-        files = md['AWS::CloudFormation::Init']['config']['files']
-        files['/etc/haproxy/haproxy.cfg']['content'] = cfg
-
-        self.nested()['LB_instance'].metadata = md
-
     def FnGetRefId(self):
         return unicode(self.name)
 
-    def FnGetAtt(self, key):
+    def _resolve_attribute(self, name):
         '''
         We don't really support any of these yet.
         '''
-        allow = ('CanonicalHostedZoneName',
-                 'CanonicalHostedZoneNameID',
-                 'DNSName',
-                 'SourceSecurityGroupName',
-                 'SourceSecurityGroupOwnerAlias')
-
-        if not key in allow:
-            raise exception.InvalidTemplateAttribute(resource=self.name,
-                                                     key=key)
-
-        if key == 'DNSName':
+        if name == 'DNSName':
             return self.get_output('PublicIp')
-        else:
+        elif name in self.attributes_schema:
+            # Not sure if we should return anything for the other attribs
+            # since they aren't really supported in any meaningful way
             return ''
 
 
