@@ -13,11 +13,9 @@
 #    under the License.
 
 
-import mox
-import unittest
-from nose.plugins.attrib import attr
+from heat.tests.common import HeatTestCase
 
-import httplib
+import requests
 import json
 from oslo.config import cfg
 
@@ -25,19 +23,17 @@ from heat.api.aws import exception
 from heat.common.wsgi import Request
 from heat.api.aws import ec2token
 
+from heat.openstack.common import importutils
 
-@attr(tag=['unit', 'api-aws', 'Ec2Token'])
-@attr(speed='fast')
-class Ec2TokenTest(unittest.TestCase):
 
+class Ec2TokenTest(HeatTestCase):
     '''
     Tests the Ec2Token middleware
     '''
-    def setUp(self):
-        self.m = mox.Mox()
 
-    def tearDown(self):
-        self.m.UnsetStubs()
+    def setUp(self):
+        super(Ec2TokenTest, self).setUp()
+        self.m.StubOutWithMock(requests, 'post')
 
     def _dummy_GET_request(self, params={}, environ={}):
         # Mangle the params dict into a query string
@@ -46,6 +42,23 @@ class Ec2TokenTest(unittest.TestCase):
         req = Request(environ)
         return req
 
+    def test_conf_get_paste(self):
+        dummy_conf = {'auth_uri': 'http://192.0.2.9/v2.0'}
+        ec2 = ec2token.EC2Token(app=None, conf=dummy_conf)
+        self.assertEqual(ec2._conf_get('auth_uri'), 'http://192.0.2.9/v2.0')
+        self.assertEqual(
+            ec2._conf_get_keystone_ec2_uri('http://192.0.2.9/v2.0'),
+            'http://192.0.2.9/v2.0/ec2tokens')
+
+    def test_conf_get_opts(self):
+        cfg.CONF.set_default('auth_uri', 'http://192.0.2.9/v2.0/',
+                             group='ec2authtoken')
+        ec2 = ec2token.EC2Token(app=None, conf={})
+        self.assertEqual(ec2._conf_get('auth_uri'), 'http://192.0.2.9/v2.0/')
+        self.assertEqual(
+            ec2._conf_get_keystone_ec2_uri('http://192.0.2.9/v2.0/'),
+            'http://192.0.2.9/v2.0/ec2tokens')
+
     def test_get_signature_param_old(self):
         params = {'Signature': 'foo'}
         dummy_req = self._dummy_GET_request(params)
@@ -168,18 +181,15 @@ class Ec2TokenTest(unittest.TestCase):
         ec2 = ec2token.EC2Token(app='xyz', conf={})
         self.assertEqual(ec2.__call__(dummy_req), 'xyz')
 
-    def _stub_http_connection(self, headers={}, params={}, response=None):
+    def _stub_http_connection(self, headers={}, params={}, response=None,
+                              req_url='http://123:5000/v2.0/ec2tokens'):
 
-        class DummyHTTPResponse:
-            resp = response
+        class DummyHTTPResponse(object):
+            text = response
 
-            def read(self):
-                return self.resp
+            def json(self):
+                return json.loads(self.text)
 
-        self.m.StubOutWithMock(httplib.HTTPConnection, '__init__')
-        httplib.HTTPConnection.__init__(mox.IgnoreArg()).AndReturn(None)
-
-        self.m.StubOutWithMock(httplib.HTTPConnection, 'request')
         body_hash = ('e3b0c44298fc1c149afbf4c8996fb9'
                      '2427ae41e4649b934ca495991b7852b855')
         req_creds = json.dumps({"ec2Credentials":
@@ -192,20 +202,11 @@ class Ec2TokenTest(unittest.TestCase):
                                  "path": "/v1",
                                  "body_hash": body_hash}})
         req_headers = {'Content-Type': 'application/json'}
-        req_path = '/foo'
-        httplib.HTTPConnection.request('POST', req_path,
-                                       body=req_creds,
-                                       headers=req_headers).AndReturn(None)
-
-        self.m.StubOutWithMock(httplib.HTTPConnection, 'getresponse')
-        httplib.HTTPConnection.getresponse().AndReturn(DummyHTTPResponse())
-
-        self.m.StubOutWithMock(httplib.HTTPConnection, 'close')
-        httplib.HTTPConnection.close().AndReturn(None)
+        requests.post(req_url, data=req_creds,
+                      headers=req_headers).AndReturn(DummyHTTPResponse())
 
     def test_call_ok(self):
-        dummy_conf = {'auth_uri': 'http://123:5000/foo',
-                      'keystone_ec2_uri': 'http://456:5000/foo'}
+        dummy_conf = {'auth_uri': 'http://123:5000/v2.0'}
         ec2 = ec2token.EC2Token(app='woot', conf=dummy_conf)
 
         auth_str = ('Authorization: foo  Credential=foo/bar, '
@@ -217,17 +218,47 @@ class Ec2TokenTest(unittest.TestCase):
                    'HTTP_AUTHORIZATION': auth_str}
         dummy_req = self._dummy_GET_request(environ=req_env)
 
-        ok_resp = json.dumps({'access': {'token': {'id': 123}}})
+        ok_resp = json.dumps({'access': {'token': {
+            'id': 123,
+            'tenant': {'name': 'tenant', 'id': 'abcd1234'}}}})
         self._stub_http_connection(headers={'Authorization': auth_str},
                                    response=ok_resp)
         self.m.ReplayAll()
         self.assertEqual(ec2.__call__(dummy_req), 'woot')
 
+        self.assertEqual('tenant', dummy_req.headers['X-Tenant-Name'])
+        self.assertEqual('abcd1234', dummy_req.headers['X-Tenant-Id'])
+        self.m.VerifyAll()
+
+    def test_call_ok_roles(self):
+        dummy_conf = {'auth_uri': 'http://123:5000/v2.0'}
+        ec2 = ec2token.EC2Token(app='woot', conf=dummy_conf)
+
+        auth_str = ('Authorization: foo  Credential=foo/bar, '
+                    'SignedHeaders=content-type;host;x-amz-date, '
+                    'Signature=xyz')
+        req_env = {'SERVER_NAME': 'heat',
+                   'SERVER_PORT': '8000',
+                   'PATH_INFO': '/v1',
+                   'HTTP_AUTHORIZATION': auth_str}
+        dummy_req = self._dummy_GET_request(environ=req_env)
+
+        ok_resp = json.dumps({'access': {
+            'token': {
+                'id': 123,
+                'tenant': {'name': 'tenant', 'id': 'abcd1234'}
+            },
+            'metadata': {'roles': ['aa', 'bb', 'cc']}}})
+        self._stub_http_connection(headers={'Authorization': auth_str},
+                                   response=ok_resp)
+        self.m.ReplayAll()
+        self.assertEqual(ec2.__call__(dummy_req), 'woot')
+
+        self.assertEqual('aa,bb,cc', dummy_req.headers['X-Roles'])
         self.m.VerifyAll()
 
     def test_call_err_tokenid(self):
-        dummy_conf = {'auth_uri': 'http://123:5000/foo',
-                      'keystone_ec2_uri': 'http://456:5000/foo'}
+        dummy_conf = {'auth_uri': 'http://123:5000/v2.0/'}
         ec2 = ec2token.EC2Token(app='woot', conf=dummy_conf)
 
         auth_str = ('Authorization: foo  Credential=foo/bar, '
@@ -250,8 +281,7 @@ class Ec2TokenTest(unittest.TestCase):
         self.m.VerifyAll()
 
     def test_call_err_signature(self):
-        dummy_conf = {'auth_uri': 'http://123:5000/foo',
-                      'keystone_ec2_uri': 'http://456:5000/foo'}
+        dummy_conf = {'auth_uri': 'http://123:5000/v2.0'}
         ec2 = ec2token.EC2Token(app='woot', conf=dummy_conf)
 
         auth_str = ('Authorization: foo  Credential=foo/bar, '
@@ -274,8 +304,7 @@ class Ec2TokenTest(unittest.TestCase):
         self.m.VerifyAll()
 
     def test_call_err_denied(self):
-        dummy_conf = {'auth_uri': 'http://123:5000/foo',
-                      'keystone_ec2_uri': 'http://456:5000/foo'}
+        dummy_conf = {'auth_uri': 'http://123:5000/v2.0'}
         ec2 = ec2token.EC2Token(app='woot', conf=dummy_conf)
 
         auth_str = ('Authorization: foo  Credential=foo/bar, '
@@ -297,8 +326,7 @@ class Ec2TokenTest(unittest.TestCase):
         self.m.VerifyAll()
 
     def test_call_ok_v2(self):
-        dummy_conf = {'auth_uri': 'http://123:5000/foo',
-                      'keystone_ec2_uri': 'http://456:5000/foo'}
+        dummy_conf = {'auth_uri': 'http://123:5000/v2.0'}
         ec2 = ec2token.EC2Token(app='woot', conf=dummy_conf)
         params = {'AWSAccessKeyId': 'foo', 'Signature': 'xyz'}
         req_env = {'SERVER_NAME': 'heat',
@@ -306,7 +334,159 @@ class Ec2TokenTest(unittest.TestCase):
                    'PATH_INFO': '/v1'}
         dummy_req = self._dummy_GET_request(params, req_env)
 
-        ok_resp = json.dumps({'access': {'token': {'id': 123}}})
+        ok_resp = json.dumps({'access': {'metadata': {}, 'token': {
+            'id': 123,
+            'tenant': {'name': 'tenant', 'id': 'abcd1234'}}}})
+        self._stub_http_connection(response=ok_resp,
+                                   params={'AWSAccessKeyId': 'foo'})
+        self.m.ReplayAll()
+        self.assertEqual(ec2.__call__(dummy_req), 'woot')
+
+        self.m.VerifyAll()
+
+    def test_call_ok_multicloud(self):
+        dummy_conf = {
+            'allowed_auth_uris': [
+                'http://123:5000/v2.0', 'http://456:5000/v2.0'],
+            'multi_cloud': True
+        }
+        ec2 = ec2token.EC2Token(app='woot', conf=dummy_conf)
+        params = {'AWSAccessKeyId': 'foo', 'Signature': 'xyz'}
+        req_env = {'SERVER_NAME': 'heat',
+                   'SERVER_PORT': '8000',
+                   'PATH_INFO': '/v1'}
+        dummy_req = self._dummy_GET_request(params, req_env)
+
+        ok_resp = json.dumps({'access': {'metadata': {}, 'token': {
+            'id': 123,
+            'tenant': {'name': 'tenant', 'id': 'abcd1234'}}}})
+        err_msg = "EC2 access key not found."
+        err_resp = json.dumps({'error': {'message': err_msg}})
+
+        # first request fails
+        self._stub_http_connection(
+            req_url='http://123:5000/v2.0/ec2tokens',
+            response=err_resp,
+            params={'AWSAccessKeyId': 'foo'})
+
+        # second request passes
+        self._stub_http_connection(
+            req_url='http://456:5000/v2.0/ec2tokens',
+            response=ok_resp,
+            params={'AWSAccessKeyId': 'foo'})
+
+        self.m.ReplayAll()
+        self.assertEqual(ec2.__call__(dummy_req), 'woot')
+
+        self.m.VerifyAll()
+
+    def test_call_err_multicloud(self):
+        dummy_conf = {
+            'allowed_auth_uris': [
+                'http://123:5000/v2.0', 'http://456:5000/v2.0'],
+            'multi_cloud': True
+        }
+        ec2 = ec2token.EC2Token(app='woot', conf=dummy_conf)
+        params = {'AWSAccessKeyId': 'foo', 'Signature': 'xyz'}
+        req_env = {'SERVER_NAME': 'heat',
+                   'SERVER_PORT': '8000',
+                   'PATH_INFO': '/v1'}
+        dummy_req = self._dummy_GET_request(params, req_env)
+
+        err_resp1 = json.dumps({})
+
+        err_msg2 = "EC2 access key not found."
+        err_resp2 = json.dumps({'error': {'message': err_msg2}})
+
+        # first request fails with HeatAccessDeniedError
+        self._stub_http_connection(
+            req_url='http://123:5000/v2.0/ec2tokens',
+            response=err_resp1,
+            params={'AWSAccessKeyId': 'foo'})
+
+        # second request fails with HeatInvalidClientTokenIdError
+        self._stub_http_connection(
+            req_url='http://456:5000/v2.0/ec2tokens',
+            response=err_resp2,
+            params={'AWSAccessKeyId': 'foo'})
+
+        self.m.ReplayAll()
+        # raised error matches last failure
+        self.assertRaises(exception.HeatInvalidClientTokenIdError,
+                          ec2.__call__, dummy_req)
+
+        self.m.VerifyAll()
+
+    def test_call_err_multicloud_none_allowed(self):
+        dummy_conf = {
+            'allowed_auth_uris': [],
+            'multi_cloud': True
+        }
+        ec2 = ec2token.EC2Token(app='woot', conf=dummy_conf)
+        params = {'AWSAccessKeyId': 'foo', 'Signature': 'xyz'}
+        req_env = {'SERVER_NAME': 'heat',
+                   'SERVER_PORT': '8000',
+                   'PATH_INFO': '/v1'}
+        dummy_req = self._dummy_GET_request(params, req_env)
+
+        self.m.ReplayAll()
+        self.assertRaises(exception.HeatAccessDeniedError,
+                          ec2.__call__, dummy_req)
+
+        self.m.VerifyAll()
+
+    def test_call_badconf_no_authuri(self):
+        ec2 = ec2token.EC2Token(app='woot', conf={})
+        params = {'AWSAccessKeyId': 'foo', 'Signature': 'xyz'}
+        req_env = {'SERVER_NAME': 'heat',
+                   'SERVER_PORT': '8000',
+                   'PATH_INFO': '/v1'}
+        dummy_req = self._dummy_GET_request(params, req_env)
+
+        self.m.ReplayAll()
+        ex = self.assertRaises(exception.HeatInternalFailureError,
+                               ec2.__call__, dummy_req)
+        self.assertEqual(str(ex), 'Service misconfigured')
+
+        self.m.VerifyAll()
+
+    def test_call_ok_auth_uri_ec2authtoken(self):
+        dummy_url = 'http://123:5000/v2.0'
+        cfg.CONF.set_default('auth_uri', dummy_url, group='ec2authtoken')
+
+        ec2 = ec2token.EC2Token(app='woot', conf={})
+        params = {'AWSAccessKeyId': 'foo', 'Signature': 'xyz'}
+        req_env = {'SERVER_NAME': 'heat',
+                   'SERVER_PORT': '8000',
+                   'PATH_INFO': '/v1'}
+        dummy_req = self._dummy_GET_request(params, req_env)
+
+        ok_resp = json.dumps({'access': {'metadata': {}, 'token': {
+            'id': 123,
+            'tenant': {'name': 'tenant', 'id': 'abcd1234'}}}})
+        self._stub_http_connection(response=ok_resp,
+                                   params={'AWSAccessKeyId': 'foo'})
+        self.m.ReplayAll()
+        self.assertEqual(ec2.__call__(dummy_req), 'woot')
+
+        self.m.VerifyAll()
+
+    def test_call_ok_auth_uri_ks_authtoken(self):
+        # Import auth_token to have keystone_authtoken settings setup.
+        importutils.import_module('keystoneclient.middleware.auth_token')
+        dummy_url = 'http://123:5000/v2.0'
+        cfg.CONF.set_override('auth_uri', dummy_url,
+                              group='keystone_authtoken')
+        ec2 = ec2token.EC2Token(app='woot', conf={})
+        params = {'AWSAccessKeyId': 'foo', 'Signature': 'xyz'}
+        req_env = {'SERVER_NAME': 'heat',
+                   'SERVER_PORT': '8000',
+                   'PATH_INFO': '/v1'}
+        dummy_req = self._dummy_GET_request(params, req_env)
+
+        ok_resp = json.dumps({'access': {'metadata': {}, 'token': {
+            'id': 123,
+            'tenant': {'name': 'tenant', 'id': 'abcd1234'}}}})
         self._stub_http_connection(response=ok_resp,
                                    params={'AWSAccessKeyId': 'foo'})
         self.m.ReplayAll()
