@@ -1,4 +1,5 @@
-#
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -14,13 +15,12 @@
 
 import copy
 
-from heat.common import exception
 from heat.common import template_format
-from heat.engine.resources.openstack.heat import cloud_watch
-from heat.engine import rsrc_defn
-from heat.engine import scheduler
+from heat.engine.resources import cloud_watch
+from heat.engine import resource
 from heat.engine import watchrule
-from heat.tests import common
+from heat.engine import scheduler
+from heat.tests.common import HeatTestCase
 from heat.tests import utils
 
 
@@ -50,16 +50,16 @@ alarm_template = '''
 '''
 
 
-class CloudWatchAlarmTest(common.HeatTestCase):
+class CloudWatchAlarmTest(HeatTestCase):
     def setUp(self):
         super(CloudWatchAlarmTest, self).setUp()
+        utils.setup_dummy_db()
 
     def create_alarm(self, t, stack, resource_name):
-        resource_defns = stack.t.resource_definitions(stack)
         rsrc = cloud_watch.CloudWatchAlarm(resource_name,
-                                           resource_defns[resource_name],
+                                           t['Resources'][resource_name],
                                            stack)
-        self.assertIsNone(rsrc.validate())
+        self.assertEqual(None, rsrc.validate())
         scheduler.TaskRunner(rsrc.create)()
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
         return rsrc
@@ -71,7 +71,7 @@ class CloudWatchAlarmTest(common.HeatTestCase):
         '''
         t = template_format.parse(alarm_template)
 
-        # short circuit the alarm's references
+        #short circuit the alarm's references
         properties = t['Resources']['MEMAlarmHigh']['Properties']
         properties['AlarmActions'] = ['a']
         properties['Dimensions'] = [{'a': 'v'}]
@@ -82,18 +82,13 @@ class CloudWatchAlarmTest(common.HeatTestCase):
 
         self.m.ReplayAll()
         rsrc = self.create_alarm(t, stack, 'MEMAlarmHigh')
-        props = copy.copy(rsrc.properties.data)
-        props.update({
-            'ComparisonOperator': 'LessThanThreshold',
-            'AlarmDescription': 'fruity',
-            'EvaluationPeriods': '2',
-            'Period': '90',
-            'Statistic': 'Maximum',
-            'Threshold': '39',
-        })
-        snippet = rsrc_defn.ResourceDefinition(rsrc.name,
-                                               rsrc.type(),
-                                               props)
+        snippet = copy.deepcopy(rsrc.parsed_template())
+        snippet['Properties']['ComparisonOperator'] = 'LessThanThreshold'
+        snippet['Properties']['AlarmDescription'] = 'fruity'
+        snippet['Properties']['EvaluationPeriods'] = '2'
+        snippet['Properties']['Period'] = '90'
+        snippet['Properties']['Statistic'] = 'Maximum'
+        snippet['Properties']['Threshold'] = '39'
 
         scheduler.TaskRunner(rsrc.update, snippet)()
 
@@ -107,7 +102,7 @@ class CloudWatchAlarmTest(common.HeatTestCase):
         '''
         t = template_format.parse(alarm_template)
 
-        # short circuit the alarm's references
+        #short circuit the alarm's references
         properties = t['Resources']['MEMAlarmHigh']['Properties']
         properties['AlarmActions'] = ['a']
         properties['Dimensions'] = [{'a': 'v'}]
@@ -118,44 +113,38 @@ class CloudWatchAlarmTest(common.HeatTestCase):
 
         self.m.ReplayAll()
         rsrc = self.create_alarm(t, stack, 'MEMAlarmHigh')
-        props = copy.copy(rsrc.properties.data)
-        props['MetricName'] = 'temp'
-        snippet = rsrc_defn.ResourceDefinition(rsrc.name,
-                                               rsrc.type(),
-                                               props)
+        snippet = copy.deepcopy(rsrc.parsed_template())
+        snippet['Properties']['MetricName'] = 'temp'
 
         updater = scheduler.TaskRunner(rsrc.update, snippet)
-        self.assertRaises(exception.UpdateReplace, updater)
+        self.assertRaises(resource.UpdateReplace, updater)
 
         scheduler.TaskRunner(rsrc.delete)()
         self.m.VerifyAll()
 
     def test_suspend_resume(self):
         t = template_format.parse(alarm_template)
-        stack_name = "test_cw_alarm_sus_res_stack"
-        stack = utils.parse_stack(t, stack_name=stack_name)
+        stack = utils.parse_stack(t)
         # the watch rule needs a valid stack_id
         stack.store()
 
         self.m.ReplayAll()
         rsrc = self.create_alarm(t, stack, 'MEMAlarmHigh')
         scheduler.TaskRunner(rsrc.suspend)()
-        self.assertEqual((rsrc.SUSPEND, rsrc.COMPLETE), rsrc.state)
-
-        self.ctx = utils.dummy_context()
+        self.assertEqual(rsrc.state, (rsrc.SUSPEND, rsrc.COMPLETE))
 
         wr = watchrule.WatchRule.load(
-            self.ctx, watch_name="%s-MEMAlarmHigh" % stack_name)
+            None, watch_name="test_stack-MEMAlarmHigh")
 
-        self.assertEqual(watchrule.WatchRule.SUSPENDED, wr.state)
+        self.assertEqual(wr.state, watchrule.WatchRule.SUSPENDED)
 
         scheduler.TaskRunner(rsrc.resume)()
-        self.assertEqual((rsrc.RESUME, rsrc.COMPLETE), rsrc.state)
+        self.assertEqual(rsrc.state, (rsrc.RESUME, rsrc.COMPLETE))
 
         wr = watchrule.WatchRule.load(
-            self.ctx, watch_name="%s-MEMAlarmHigh" % stack_name)
+            None, watch_name="test_stack-MEMAlarmHigh")
 
-        self.assertEqual(watchrule.WatchRule.NODATA, wr.state)
+        self.assertEqual(wr.state, watchrule.WatchRule.NODATA)
 
         scheduler.TaskRunner(rsrc.delete)()
         self.m.VerifyAll()
