@@ -12,17 +12,19 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import unittest
-import mox
-
-from nose.plugins.attrib import attr
 
+from heat.engine import environment
 from heat.tests.v1_1 import fakes
 from heat.engine.resources import instance as instances
 from heat.engine.resources import network_interface as network_interfaces
+from heat.engine.resources import nova_utils
 from heat.common import template_format
 from heat.engine import parser
+from heat.engine import scheduler
 from heat.openstack.common import uuidutils
+from heat.tests.common import HeatTestCase
+from heat.tests import utils
+
 
 wp_template = '''
 {
@@ -60,6 +62,7 @@ wp_template = '''
 }
 '''
 
+
 wp_template_with_nic = '''
 {
   "AWSTemplateFormatVersion" : "2010-09-09",
@@ -106,7 +109,7 @@ wp_template_with_nic = '''
 '''
 
 
-class FakeQuantum(object):
+class FakeNeutron(object):
 
     def show_subnet(self, subnet, **_params):
         return {
@@ -121,7 +124,6 @@ class FakeQuantum(object):
                 'cidr': '10.10.0.0/24',
                 'id': '4156c7a5-e8c4-4aff-a6e1-8f3c7bc83861',
                 'enable_dhcp': False,
-
             }}
 
     def create_port(self, body=None):
@@ -142,22 +144,21 @@ class FakeQuantum(object):
             }}
 
 
-@attr(tag=['unit', 'resource', 'instance'])
-@attr(speed='fast')
-class instancesTest(unittest.TestCase):
+class instancesTest(HeatTestCase):
     def setUp(self):
-        self.m = mox.Mox()
+        super(instancesTest, self).setUp()
         self.fc = fakes.FakeClient()
+        utils.setup_dummy_db()
 
     def _create_test_instance(self, return_server, name):
-        stack_name = '%s_stack' % name
+        stack_name = '%s_s' % name
         t = template_format.parse(wp_template)
         template = parser.Template(t)
         kwargs = {'KeyName': 'test',
                   'InstanceType': 'm1.large',
                   'SubnetId': '4156c7a5-e8c4-4aff-a6e1-8f3c7bc83861'}
-        params = parser.Parameters(stack_name, template, kwargs)
-        stack = parser.Stack(None, stack_name, template, params,
+        stack = parser.Stack(utils.dummy_context(), stack_name, template,
+                             environment.Environment(kwargs),
                              stack_id=uuidutils.generate_uuid())
 
         t['Resources']['WebServer']['Properties']['ImageId'] = 'CentOS 5.2'
@@ -167,19 +168,20 @@ class instancesTest(unittest.TestCase):
         self.m.StubOutWithMock(instance, 'nova')
         instance.nova().MultipleTimes().AndReturn(self.fc)
 
-        self.m.StubOutWithMock(instance, 'quantum')
-        instance.quantum().MultipleTimes().AndReturn(FakeQuantum())
+        self.m.StubOutWithMock(instance, 'neutron')
+        instance.neutron().MultipleTimes().AndReturn(FakeNeutron())
 
         instance.t = instance.stack.resolve_runtime_data(instance.t)
 
         # need to resolve the template functions
-        server_userdata = instance._build_userdata(
+        server_userdata = nova_utils.build_userdata(
+            instance,
             instance.t['Properties']['UserData'])
-
+        instance.mime_string = server_userdata
         self.m.StubOutWithMock(self.fc.servers, 'create')
         self.fc.servers.create(
             image=1, flavor=3, key_name='test',
-            name='%s.%s' % (stack_name, instance.name),
+            name=utils.PhysName(stack_name, instance.name),
             security_groups=None,
             userdata=server_userdata, scheduler_hints=None, meta=None,
             nics=[{'port-id': '64d913c1-bcb1-42d2-8f0a-9593dbcaf251'}],
@@ -187,18 +189,18 @@ class instancesTest(unittest.TestCase):
                 return_server)
         self.m.ReplayAll()
 
-        self.assertEqual(instance.create(), None)
+        scheduler.TaskRunner(instance.create)()
         return instance
 
     def _create_test_instance_with_nic(self, return_server, name):
-        stack_name = '%s_stack' % name
+        stack_name = '%s_s' % name
         t = template_format.parse(wp_template_with_nic)
         template = parser.Template(t)
         kwargs = {'KeyName': 'test',
                   'InstanceType': 'm1.large',
                   'SubnetId': '4156c7a5-e8c4-4aff-a6e1-8f3c7bc83861'}
-        params = parser.Parameters(stack_name, template, kwargs)
-        stack = parser.Stack(None, stack_name, template, params,
+        stack = parser.Stack(utils.dummy_context(), stack_name, template,
+                             environment.Environment(kwargs),
                              stack_id=uuidutils.generate_uuid())
 
         t['Resources']['WebServer']['Properties']['ImageId'] = 'CentOS 5.2'
@@ -210,8 +212,8 @@ class instancesTest(unittest.TestCase):
         instance = instances.Instance('%s_name' % name,
                                       t['Resources']['WebServer'], stack)
 
-        self.m.StubOutWithMock(nic, 'quantum')
-        nic.quantum().MultipleTimes().AndReturn(FakeQuantum())
+        self.m.StubOutWithMock(nic, 'neutron')
+        nic.neutron().MultipleTimes().AndReturn(FakeNeutron())
 
         self.m.StubOutWithMock(instance, 'nova')
         instance.nova().MultipleTimes().AndReturn(self.fc)
@@ -220,12 +222,14 @@ class instancesTest(unittest.TestCase):
         instance.t = instance.stack.resolve_runtime_data(instance.t)
 
         # need to resolve the template functions
-        server_userdata = instance._build_userdata(
+        server_userdata = nova_utils.build_userdata(
+            instance,
             instance.t['Properties']['UserData'])
+        instance.mime_string = server_userdata
         self.m.StubOutWithMock(self.fc.servers, 'create')
         self.fc.servers.create(
             image=1, flavor=3, key_name='test',
-            name='%s.%s' % (stack_name, instance.name),
+            name=utils.PhysName(stack_name, instance.name),
             security_groups=None,
             userdata=server_userdata, scheduler_hints=None, meta=None,
             nics=[{'port-id': '64d913c1-bcb1-42d2-8f0a-9593dbcaf251'}],
@@ -234,16 +238,16 @@ class instancesTest(unittest.TestCase):
         self.m.ReplayAll()
 
         # create network interface
-        self.assertEqual(nic.create(), None)
+        scheduler.TaskRunner(nic.create)()
         stack.resources["nic1"] = nic
 
-        self.assertEqual(instance.create(), None)
+        scheduler.TaskRunner(instance.create)()
         return instance
 
     def test_instance_create(self):
         return_server = self.fc.servers.list()[1]
         instance = self._create_test_instance(return_server,
-                                              'test_instance_create')
+                                              'in_create')
         # this makes sure the auto increment worked on instance creation
         self.assertTrue(instance.id > 0)
 
@@ -258,7 +262,7 @@ class instancesTest(unittest.TestCase):
     def test_instance_create_with_nic(self):
         return_server = self.fc.servers.list()[1]
         instance = self._create_test_instance_with_nic(
-            return_server, 'test_instance_create_with_network_interface')
+            return_server, 'in_create_wnic')
 
         # this makes sure the auto increment worked on instance creation
         self.assertTrue(instance.id > 0)
