@@ -1,4 +1,5 @@
-#
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -11,13 +12,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from heat.tests.v1_1 import fakes
+from heat.engine.resources import instance as instances
+from heat.engine.resources import nova_utils
 from heat.common import template_format
-from heat.engine.clients.os import glance
-from heat.engine.clients.os import nova
-from heat.engine.resources.aws.ec2 import instance as instances
 from heat.engine import scheduler
-from heat.tests import common
-from heat.tests.openstack.nova import fakes as fakes_nova
+from heat.tests.common import HeatTestCase
 from heat.tests import utils
 
 
@@ -40,10 +40,11 @@ nokey_template = '''
 '''
 
 
-class nokeyTest(common.HeatTestCase):
+class nokeyTest(HeatTestCase):
     def setUp(self):
         super(nokeyTest, self).setUp()
-        self.fc = fakes_nova.FakeClient()
+        self.fc = fakes.FakeClient()
+        utils.setup_dummy_db()
 
     def test_nokey_create(self):
 
@@ -52,42 +53,29 @@ class nokeyTest(common.HeatTestCase):
         stack = utils.parse_stack(t, stack_name=stack_name)
 
         t['Resources']['WebServer']['Properties']['ImageId'] = 'CentOS 5.2'
-        t['Resources']['WebServer']['Properties'][
-            'InstanceType'] = '256 MB Server'
-        resource_defns = stack.t.resource_definitions(stack)
+        t['Resources']['WebServer']['Properties']['InstanceType'] = \
+            '256 MB Server'
         instance = instances.Instance('create_instance_name',
-                                      resource_defns['WebServer'], stack)
+                                      t['Resources']['WebServer'], stack)
 
-        self.m.StubOutWithMock(nova.NovaClientPlugin, '_create')
-        nova.NovaClientPlugin._create().AndReturn(self.fc)
-        self.m.StubOutWithMock(glance.GlanceClientPlugin,
-                               'find_image_by_name_or_id')
-        glance.GlanceClientPlugin.find_image_by_name_or_id(
-            'CentOS 5.2').MultipleTimes().AndReturn(1)
+        self.m.StubOutWithMock(instance, 'nova')
+        instance.nova().MultipleTimes().AndReturn(self.fc)
+
+        instance.t = instance.stack.resolve_runtime_data(instance.t)
 
         # need to resolve the template functions
-        metadata = instance.metadata_get()
-        server_userdata = instance.client_plugin().build_userdata(
-            metadata,
-            instance.properties['UserData'],
-            'ec2-user')
-        self.m.StubOutWithMock(nova.NovaClientPlugin, 'build_userdata')
-        nova.NovaClientPlugin.build_userdata(
-            metadata,
-            instance.properties['UserData'],
-            'ec2-user').AndReturn(server_userdata)
-
+        server_userdata = nova_utils.build_userdata(
+            instance,
+            instance.t['Properties']['UserData'])
+        instance.mime_string = server_userdata
         self.m.StubOutWithMock(self.fc.servers, 'create')
         self.fc.servers.create(
             image=1, flavor=1, key_name=None,
             name=utils.PhysName(stack_name, instance.name),
             security_groups=None,
             userdata=server_userdata, scheduler_hints=None,
-            meta=None, nics=None, availability_zone=None,
-            block_device_mapping=None).AndReturn(
+            meta=None, nics=None, availability_zone=None).AndReturn(
                 self.fc.servers.list()[1])
         self.m.ReplayAll()
 
         scheduler.TaskRunner(instance.create)()
-
-        self.m.VerifyAll()
