@@ -13,48 +13,61 @@
 #    under the License.
 
 
-import unittest
-from nose.plugins.attrib import attr
+import testtools
 import json
 
+from heat.common import exception
 from heat.engine import parameters
+from heat.engine import template
 
 
-@attr(tag=['unit', 'parameters'])
-@attr(speed='fast')
-class ParameterTest(unittest.TestCase):
+class ParameterTest(testtools.TestCase):
+
+    def new_parameter(self, name, schema, value=None,
+                      validate_value=True):
+        tmpl = template.Template({template.PARAMETERS: {name:
+                                                        schema}})
+        schema = tmpl.param_schemata()[name]
+        return parameters.Parameter(name, schema, value,
+                                    validate_value)
+
     def test_new_string(self):
-        p = parameters.Parameter('p', {'Type': 'String'})
+        p = self.new_parameter('p', {'Type': 'String'}, validate_value=False)
         self.assertTrue(isinstance(p, parameters.StringParam))
 
     def test_new_number(self):
-        p = parameters.Parameter('p', {'Type': 'Number'})
+        p = self.new_parameter('p', {'Type': 'Number'}, validate_value=False)
         self.assertTrue(isinstance(p, parameters.NumberParam))
 
     def test_new_list(self):
-        p = parameters.Parameter('p', {'Type': 'CommaDelimitedList'})
+        p = self.new_parameter('p', {'Type': 'CommaDelimitedList'},
+                               validate_value=False)
         self.assertTrue(isinstance(p, parameters.CommaDelimitedListParam))
 
+    def test_new_json(self):
+        p = self.new_parameter('p', {'Type': 'Json'}, validate_value=False)
+        self.assertTrue(isinstance(p, parameters.JsonParam))
+
     def test_new_bad_type(self):
-        self.assertRaises(ValueError, parameters.Parameter,
-                          'p', {'Type': 'List'})
+        self.assertRaises(ValueError, self.new_parameter, 'p',
+                          {'Type': 'List'})
 
     def test_new_no_type(self):
-        self.assertRaises(KeyError, parameters.Parameter,
+        self.assertRaises(KeyError, self.new_parameter,
                           'p', {'Default': 'blarg'})
 
     def test_default_no_override(self):
-        p = parameters.Parameter('defaulted', {'Type': 'String',
-                                               'Default': 'blarg'})
+        p = self.new_parameter('defaulted', {'Type': 'String',
+                                             'Default': 'blarg'})
         self.assertTrue(p.has_default())
         self.assertEqual(p.default(), 'blarg')
         self.assertEqual(p.value(), 'blarg')
 
     def test_default_override(self):
-        p = parameters.Parameter('defaulted',
-                                 {'Type': 'String',
-                                  'Default': 'blarg'},
-                                 'wibble')
+        p = self.new_parameter('defaulted',
+                               {'Type': 'String',
+                                'Default': 'blarg'},
+                               'wibble')
         self.assertTrue(p.has_default())
         self.assertEqual(p.default(), 'blarg')
         self.assertEqual(p.value(), 'wibble')
@@ -65,7 +78,7 @@ class ParameterTest(unittest.TestCase):
                   'ConstraintDescription': 'wibble',
                   'Default': 'bar'}
         try:
-            parameters.Parameter('p', schema, 'foo')
+            self.new_parameter('p', schema, 'foo')
         except ValueError as ve:
             msg = str(ve)
             self.assertNotEqual(msg.find('wibble'), -1)
@@ -73,44 +86,45 @@ class ParameterTest(unittest.TestCase):
             self.fail('ValueError not raised')
 
     def test_no_echo_true(self):
-        p = parameters.Parameter('anechoic',
-                                 {'Type': 'String',
-                                 'NoEcho': 'true'},
-                                 'wibble')
+        p = self.new_parameter('anechoic',
+                               {'Type': 'String',
+                                'NoEcho': 'true'},
+                               'wibble')
         self.assertTrue(p.no_echo())
         self.assertNotEqual(str(p), 'wibble')
 
     def test_no_echo_true_caps(self):
-        p = parameters.Parameter('anechoic',
-                                 {'Type': 'String',
-                                 'NoEcho': 'TrUe'},
-                                 'wibble')
+        p = self.new_parameter('anechoic',
+                               {'Type': 'String',
+                                'NoEcho': 'TrUe'},
+                               'wibble')
         self.assertTrue(p.no_echo())
         self.assertNotEqual(str(p), 'wibble')
 
     def test_no_echo_false(self):
-        p = parameters.Parameter('echoic',
-                                 {'Type': 'String',
-                                 'NoEcho': 'false'},
-                                 'wibble')
+        p = self.new_parameter('echoic',
+                               {'Type': 'String',
+                                'NoEcho': 'false'},
+                               'wibble')
         self.assertFalse(p.no_echo())
         self.assertEqual(str(p), 'wibble')
 
     def test_description(self):
         description = 'Description of the parameter'
-        p = parameters.Parameter('p', {'Type': 'String',
-                                       'Description': description})
+        p = self.new_parameter('p', {'Type': 'String',
+                                     'Description': description},
+                               validate_value=False)
         self.assertEqual(p.description(), description)
 
     def test_no_description(self):
-        p = parameters.Parameter('p', {'Type': 'String'})
+        p = self.new_parameter('p', {'Type': 'String'}, validate_value=False)
         self.assertEqual(p.description(), '')
 
     def test_string_len_good(self):
         schema = {'Type': 'String',
                   'MinLength': '3',
                   'MaxLength': '3'}
-        p = parameters.Parameter('p', schema, 'foo')
+        p = self.new_parameter('p', schema, 'foo')
         self.assertEqual(p.value(), 'foo')
 
     def test_string_underflow(self):
@@ -118,7 +132,7 @@ class ParameterTest(unittest.TestCase):
                   'ConstraintDescription': 'wibble',
                   'MinLength': '4'}
         try:
-            parameters.Parameter('p', schema, 'foo')
+            self.new_parameter('p', schema, 'foo')
         except ValueError as ve:
             msg = str(ve)
             self.assertNotEqual(msg.find('wibble'), -1)
@@ -130,7 +144,7 @@ class ParameterTest(unittest.TestCase):
                   'ConstraintDescription': 'wibble',
                   'MaxLength': '2'}
         try:
-            parameters.Parameter('p', schema, 'foo')
+            self.new_parameter('p', schema, 'foo')
         except ValueError as ve:
             msg = str(ve)
             self.assertNotEqual(msg.find('wibble'), -1)
@@ -140,7 +154,7 @@ class ParameterTest(unittest.TestCase):
     def test_string_pattern_good(self):
         schema = {'Type': 'String',
                   'AllowedPattern': '[a-z]*'}
-        p = parameters.Parameter('p', schema, 'foo')
+        p = self.new_parameter('p', schema, 'foo')
         self.assertEqual(p.value(), 'foo')
 
     def test_string_pattern_bad_prefix(self):
@@ -148,7 +162,7 @@ class ParameterTest(unittest.TestCase):
                   'ConstraintDescription': 'wibble',
                   'AllowedPattern': '[a-z]*'}
         try:
-            parameters.Parameter('p', schema, '1foo')
+            self.new_parameter('p', schema, '1foo')
         except ValueError as ve:
             msg = str(ve)
             self.assertNotEqual(msg.find('wibble'), -1)
@@ -160,7 +174,7 @@ class ParameterTest(unittest.TestCase):
                   'ConstraintDescription': 'wibble',
                   'AllowedPattern': '[a-z]*'}
         try:
-            parameters.Parameter('p', schema, 'foo1')
+            self.new_parameter('p', schema, 'foo1')
         except ValueError as ve:
             msg = str(ve)
             self.assertNotEqual(msg.find('wibble'), -1)
@@ -170,7 +184,7 @@ class ParameterTest(unittest.TestCase):
     def test_string_value_list_good(self):
         schema = {'Type': 'String',
                   'AllowedValues': ['foo', 'bar', 'baz']}
-        p = parameters.Parameter('p', schema, 'bar')
+        p = self.new_parameter('p', schema, 'bar')
         self.assertEqual(p.value(), 'bar')
 
     def test_string_value_list_bad(self):
@@ -178,7 +192,7 @@ class ParameterTest(unittest.TestCase):
                   'ConstraintDescription': 'wibble',
                   'AllowedValues': ['foo', 'bar', 'baz']}
         try:
-            parameters.Parameter('p', schema, 'blarg')
+            self.new_parameter('p', schema, 'blarg')
         except ValueError as ve:
             msg = str(ve)
             self.assertNotEqual(msg.find('wibble'), -1)
@@ -189,22 +203,22 @@ class ParameterTest(unittest.TestCase):
         schema = {'Type': 'Number',
                   'MinValue': '3',
                   'MaxValue': '3'}
-        p = parameters.Parameter('p', schema, '3')
-        self.assertEqual(p.value(), '3')
+        p = self.new_parameter('p', schema, '3')
+        self.assertEqual(p.value(), 3)
 
     def test_number_float_good(self):
         schema = {'Type': 'Number',
                   'MinValue': '3.0',
-                  'MaxValue': '3.0'}
-        p = parameters.Parameter('p', schema, '3.0')
-        self.assertEqual(p.value(), '3.0')
+                  'MaxValue': '4.0'}
+        p = self.new_parameter('p', schema, '3.5')
+        self.assertEqual(p.value(), 3.5)
 
     def test_number_low(self):
         schema = {'Type': 'Number',
                   'ConstraintDescription': 'wibble',
                   'MinValue': '4'}
         try:
-            parameters.Parameter('p', schema, '3')
+            self.new_parameter('p', schema, '3')
         except ValueError as ve:
             msg = str(ve)
             self.assertNotEqual(msg.find('wibble'), -1)
@@ -216,7 +230,7 @@ class ParameterTest(unittest.TestCase):
                   'ConstraintDescription': 'wibble',
                   'MaxValue': '2'}
         try:
-            parameters.Parameter('p', schema, '3')
+            self.new_parameter('p', schema, '3')
         except ValueError as ve:
             msg = str(ve)
             self.assertNotEqual(msg.find('wibble'), -1)
@@ -226,15 +240,15 @@ class ParameterTest(unittest.TestCase):
     def test_number_value_list_good(self):
         schema = {'Type': 'Number',
                   'AllowedValues': ['1', '3', '5']}
-        p = parameters.Parameter('p', schema, '5')
-        self.assertEqual(p.value(), '5')
+        p = self.new_parameter('p', schema, '5')
+        self.assertEqual(p.value(), 5)
 
     def test_number_value_list_bad(self):
         schema = {'Type': 'Number',
                   'ConstraintDescription': 'wibble',
                   'AllowedValues': ['1', '3', '5']}
         try:
-            parameters.Parameter('p', schema, '2')
+            self.new_parameter('p', schema, '2')
         except ValueError as ve:
             msg = str(ve)
             self.assertNotEqual(msg.find('wibble'), -1)
@@ -244,21 +258,121 @@ class ParameterTest(unittest.TestCase):
     def test_list_value_list_good(self):
         schema = {'Type': 'CommaDelimitedList',
                   'AllowedValues': ['foo', 'bar', 'baz']}
-        p = parameters.Parameter('p', schema, 'baz,foo,bar')
-        self.assertEqual(p.value(), 'baz,foo,bar')
+        p = self.new_parameter('p', schema, 'baz,foo,bar')
+        self.assertEqual(p.value(), 'baz,foo,bar'.split(','))
+        schema['Default'] = []
+        p = self.new_parameter('p', schema)
+        self.assertEqual(p.value(), [])
+        schema['Default'] = 'baz,foo,bar'
+        p = self.new_parameter('p', schema)
+        self.assertEqual(p.value(), 'baz,foo,bar'.split(','))
 
     def test_list_value_list_bad(self):
         schema = {'Type': 'CommaDelimitedList',
                   'ConstraintDescription': 'wibble',
                   'AllowedValues': ['foo', 'bar', 'baz']}
         try:
-            parameters.Parameter('p', schema, 'foo,baz,blarg')
+            self.new_parameter('p', schema, 'foo,baz,blarg')
         except ValueError as ve:
             msg = str(ve)
             self.assertNotEqual(msg.find('wibble'), -1)
         else:
             self.fail('ValueError not raised')
 
+    def test_map_value(self):
+        '''Happy path for value thats already a map.'''
+        schema = {'Type': 'Json'}
+        val = {"foo": "bar", "items": [1, 2, 3]}
+        val_s = json.dumps(val)
+        p = self.new_parameter('p', schema, val)
+        self.assertEqual(val_s, p.value())
+        self.assertEqual(val, p.parsed)
+
+    def test_map_value_bad(self):
+        '''Map value is not JSON parsable.'''
+        schema = {'Type': 'Json',
+                  'ConstraintDescription': 'wibble'}
+        val = {"foo": "bar", "not_json": len}
+        try:
+            self.new_parameter('p', schema, val)
+        except ValueError as verr:
+            self.assertIn('Value must be valid JSON', str(verr))
+        else:
+            self.fail("Value error not raised")
+
+    def test_map_value_parse(self):
+        '''Happy path for value that's a string.'''
+        schema = {'Type': 'Json'}
+        val = {"foo": "bar", "items": [1, 2, 3]}
+        val_s = json.dumps(val)
+        p = self.new_parameter('p', schema, val_s)
+        self.assertEqual(val_s, p.value())
+        self.assertEqual(val, p.parsed)
+
+    def test_map_value_bad_parse(self):
+        '''Test value error for unparsable string value.'''
+        schema = {'Type': 'Json',
+                  'ConstraintDescription': 'wibble'}
+        val = "I am not a map"
+        try:
+            self.new_parameter('p', schema, val)
+        except ValueError as verr:
+            self.assertIn('Value must be valid JSON', str(verr))
+        else:
+            self.fail("Value error not raised")
+
+    def test_map_values_good(self):
+        '''Happy path for map keys.'''
+        schema = {'Type': 'Json',
+                  'AllowedValues': ["foo", "bar", "baz"]}
+        val = {"foo": "bar", "baz": [1, 2, 3]}
+        val_s = json.dumps(val)
+        p = self.new_parameter('p', schema, val_s)
+        self.assertEqual(val_s, p.value())
+        self.assertEqual(val, p.parsed)
+
+    def test_map_values_bad(self):
+        '''Test failure of invalid map keys.'''
+        schema = {'Type': 'Json',
+                  'AllowedValues': ["foo", "bar", "baz"]}
+        val = {"foo": "bar", "items": [1, 2, 3]}
+        try:
+            self.new_parameter('p', schema, val)
+        except ValueError as verr:
+            self.assertIn("items", str(verr))
+        else:
+            self.fail("Value error not raised")
+
+    def test_map_underrun(self):
+        '''Test map length under MIN_LEN.'''
+        schema = {'Type': 'Json',
+                  'MinLength': 3}
+        val = {"foo": "bar", "items": [1, 2, 3]}
+        try:
+            self.new_parameter('p', schema, val)
+        except ValueError as verr:
+            self.assertIn('underflows', str(verr))
+        else:
+            self.fail("Value error not raised")
+
+    def test_map_overrun(self):
+        '''Test map length over MAX_LEN.'''
+        schema = {'Type': 'Json',
+                  'MaxLength': 1}
+        val = {"foo": "bar", "items": [1, 2, 3]}
+        try:
+            self.new_parameter('p', schema, val)
+        except ValueError as verr:
+            self.assertIn('overflows', str(verr))
+        else:
+            self.fail("Value error not raised")
+
+    def test_missing_param(self):
+        '''Test missing user parameter.'''
+        self.assertRaises(exception.UserParameterMissing,
+                          self.new_parameter, 'p',
+                          {'Type': 'String'})
+
 
 params_schema = json.loads('''{
   "Parameters" : {
@@ -271,56 +385,52 @@ params_schema = json.loads('''{
 }''')
 
 
-@attr(tag=['unit', 'parameters'])
-@attr(speed='fast')
-class ParametersTest(unittest.TestCase):
+class ParametersTest(testtools.TestCase):
+    def new_parameters(self, stack_name, tmpl, user_params={}, stack_id=None,
+                       validate_value=True):
+        tmpl = template.Template(tmpl)
+        return parameters.Parameters(stack_name, tmpl, user_params, stack_id,
+                                     validate_value)
+
     def test_pseudo_params(self):
-        params = parameters.Parameters('test_stack', {"Parameters": {}})
+        params = self.new_parameters('test_stack', {"Parameters": {}})
 
         self.assertEqual(params['AWS::StackName'], 'test_stack')
         self.assertEqual(params['AWS::StackId'], 'None')
         self.assertTrue('AWS::Region' in params)
 
     def test_pseudo_param_stackid(self):
-        params = parameters.Parameters('test_stack', {'Parameters': {}},
-                                       stack_id='123::foo')
+        params = self.new_parameters('test_stack', {'Parameters': {}},
+                                     stack_id='123::foo')
 
         self.assertEqual(params['AWS::StackId'], '123::foo')
         params.set_stack_id('456::bar')
         self.assertEqual(params['AWS::StackId'], '456::bar')
 
-    def test_user_param(self):
-        user_params = {'User': 'wibble'}
-        params = parameters.Parameters('test', params_schema, user_params)
-        self.assertEqual(params.user_parameters(), user_params)
-
-    def test_user_param_nonexist(self):
-        params = parameters.Parameters('test', params_schema)
-        self.assertEqual(params.user_parameters(), {})
-
     def test_schema_invariance(self):
-        params1 = parameters.Parameters('test', params_schema,
-                                        {'Defaulted': 'wibble'})
+        params1 = self.new_parameters('test', params_schema,
+                                      {'User': 'foo',
+                                       'Defaulted': 'wibble'})
         self.assertEqual(params1['Defaulted'], 'wibble')
 
-        params2 = parameters.Parameters('test', params_schema)
+        params2 = self.new_parameters('test', params_schema, {'User': 'foo'})
         self.assertEqual(params2['Defaulted'], 'foobar')
 
     def test_to_dict(self):
         template = {'Parameters': {'Foo': {'Type': 'String'},
                                    'Bar': {'Type': 'Number', 'Default': '42'}}}
-        params = parameters.Parameters('test_params', template, {'Foo': 'foo'})
+        params = self.new_parameters('test_params', template, {'Foo': 'foo'})
 
         as_dict = dict(params)
         self.assertEqual(as_dict['Foo'], 'foo')
-        self.assertEqual(as_dict['Bar'], '42')
+        self.assertEqual(as_dict['Bar'], 42)
         self.assertEqual(as_dict['AWS::StackName'], 'test_params')
         self.assertTrue('AWS::Region' in as_dict)
 
     def test_map(self):
         template = {'Parameters': {'Foo': {'Type': 'String'},
                                    'Bar': {'Type': 'Number', 'Default': '42'}}}
-        params = parameters.Parameters('test_params', template, {'Foo': 'foo'})
+        params = self.new_parameters('test_params', template, {'Foo': 'foo'})
 
         expected = {'Foo': False,
                     'Bar': True,
@@ -333,8 +443,8 @@ class ParametersTest(unittest.TestCase):
     def test_map_str(self):
         template = {'Parameters': {'Foo': {'Type': 'String'},
                                    'Bar': {'Type': 'Number'}}}
-        params = parameters.Parameters('test_params', template, {
-            'Foo': 'foo', 'Bar': 42})
+        params = self.new_parameters('test_params', template,
+                                     {'Foo': 'foo', 'Bar': '42'})
 
         expected = {'Foo': 'foo',
                     'Bar': '42',
@@ -343,3 +453,28 @@ class ParametersTest(unittest.TestCase):
                     'AWS::StackName': 'test_params'}
 
         self.assertEqual(params.map(str), expected)
+
+    def test_unknown_params(self):
+        user_params = {'Foo': 'wibble'}
+        self.assertRaises(exception.UnknownUserParameter,
+                          self.new_parameters,
+                          'test',
+                          params_schema,
+                          user_params)
+
+    def test_missing_params(self):
+        user_params = {}
+        self.assertRaises(exception.UserParameterMissing,
+                          self.new_parameters,
+                          'test',
+                          params_schema,
+                          user_params)
+
+    def test_missing_attribute_params(self):
+        params = {'Parameters': {'Foo': {'Type': 'String'},
+                                 'NoAttr': 'No attribute.',
+                                 'Bar': {'Type': 'Number', 'Default': '1'}}}
+        self.assertRaises(exception.InvalidTemplateParameter,
+                          self.new_parameters,
+                          'test',
+                          params)
