@@ -12,26 +12,26 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-
-import mox
-import uuid
 import datetime
+import time
 import json
 
-import eventlet
-from nose.plugins.attrib import attr
 from oslo.config import cfg
-import unittest
 
+from heat.openstack.common import uuidutils
+
+from heat.tests.common import HeatTestCase
 from heat.tests import fakes
-from heat.tests.utils import stack_delete_after
+from heat.tests import utils
 
-import heat.db as db_api
+import heat.db.api as db_api
 from heat.common import template_format
 from heat.common import identifier
+from heat.engine import environment
 from heat.engine import parser
+from heat.engine import resource
+from heat.engine import scheduler
 from heat.engine.resources import wait_condition as wc
-from heat.common import context
 
 test_template_waitcondition = '''
 {
@@ -75,39 +75,42 @@ test_template_wc_count = '''
 '''
 
 
-@attr(tag=['unit', 'resource', 'WaitCondition'])
-@attr(speed='slow')
-class WaitConditionTest(unittest.TestCase):
+class WaitConditionTest(HeatTestCase):
+
     def setUp(self):
-        self.m = mox.Mox()
+        super(WaitConditionTest, self).setUp()
+        utils.setup_dummy_db()
         self.m.StubOutWithMock(wc.WaitConditionHandle,
                                'get_status')
-        self.m.StubOutWithMock(wc.WaitCondition,
-                               '_create_timeout')
-        self.m.StubOutWithMock(eventlet, 'sleep')
 
         cfg.CONF.set_default('heat_waitcondition_server_url',
-                             'http://127.0.0.1:8000/v1/waitcondition')
+                             'http://server.test:8000/v1/waitcondition')
 
         self.fc = fakes.FakeKeystoneClient()
 
     def tearDown(self):
-        self.m.UnsetStubs()
+        super(WaitConditionTest, self).tearDown()
+        utils.reset_dummy_db()
 
     # Note tests creating a stack should be decorated with @stack_delete_after
     # to ensure the stack is properly cleaned up
-    def create_stack(self, stack_name='test_stack',
+    def create_stack(self, stack_id=None,
                      template=test_template_waitcondition, params={},
                      stub=True):
         temp = template_format.parse(template)
         template = parser.Template(temp)
-        parameters = parser.Parameters(stack_name, template, params)
-        ctx = context.get_admin_context()
-        ctx.tenant_id = 'test_tenant'
-        stack = parser.Stack(ctx, stack_name, template, parameters,
+        ctx = utils.dummy_context(tenant_id='test_tenant')
+        stack = parser.Stack(ctx, 'test_stack', template,
+                             environment.Environment(params),
                              disable_rollback=True)
 
-        self.stack_id = stack.store()
+        # Stub out the stack ID so we have a known value
+        if stack_id is None:
+            stack_id = uuidutils.generate_uuid()
+
+        self.stack_id = stack_id
+        with utils.UUIDStub(self.stack_id):
+            stack.store()
 
         if stub:
             self.m.StubOutWithMock(wc.WaitConditionHandle, 'keystone')
@@ -121,62 +124,53 @@ class WaitConditionTest(unittest.TestCase):
 
         return stack
 
-    @stack_delete_after
+    @utils.stack_delete_after
     def test_post_success_to_handle(self):
         self.stack = self.create_stack()
-        wc.WaitCondition._create_timeout().AndReturn(eventlet.Timeout(5))
         wc.WaitConditionHandle.get_status().AndReturn([])
-        eventlet.sleep(1).AndReturn(None)
         wc.WaitConditionHandle.get_status().AndReturn([])
-        eventlet.sleep(1).AndReturn(None)
         wc.WaitConditionHandle.get_status().AndReturn(['SUCCESS'])
 
         self.m.ReplayAll()
 
         self.stack.create()
 
-        resource = self.stack.resources['WaitForTheHandle']
-        self.assertEqual(resource.state,
-                         'CREATE_COMPLETE')
+        rsrc = self.stack.resources['WaitForTheHandle']
+        self.assertEqual(rsrc.state,
+                         (rsrc.CREATE, rsrc.COMPLETE))
 
         r = db_api.resource_get_by_name_and_stack(None, 'WaitHandle',
                                                   self.stack.id)
         self.assertEqual(r.name, 'WaitHandle')
         self.m.VerifyAll()
 
-    @stack_delete_after
+    @utils.stack_delete_after
     def test_post_failure_to_handle(self):
         self.stack = self.create_stack()
-        wc.WaitCondition._create_timeout().AndReturn(eventlet.Timeout(5))
         wc.WaitConditionHandle.get_status().AndReturn([])
-        eventlet.sleep(1).AndReturn(None)
         wc.WaitConditionHandle.get_status().AndReturn([])
-        eventlet.sleep(1).AndReturn(None)
         wc.WaitConditionHandle.get_status().AndReturn(['FAILURE'])
 
         self.m.ReplayAll()
 
         self.stack.create()
 
-        resource = self.stack.resources['WaitForTheHandle']
-        self.assertEqual(resource.state,
-                         'CREATE_FAILED')
+        rsrc = self.stack.resources['WaitForTheHandle']
+        self.assertEqual(rsrc.state, (rsrc.CREATE, rsrc.FAILED))
+        reason = rsrc.status_reason
+        self.assertTrue(reason.startswith('WaitConditionFailure:'))
 
         r = db_api.resource_get_by_name_and_stack(None, 'WaitHandle',
                                                   self.stack.id)
         self.assertEqual(r.name, 'WaitHandle')
         self.m.VerifyAll()
 
-    @stack_delete_after
+    @utils.stack_delete_after
     def test_post_success_to_handle_count(self):
         self.stack = self.create_stack(template=test_template_wc_count)
-        wc.WaitCondition._create_timeout().AndReturn(eventlet.Timeout(5))
         wc.WaitConditionHandle.get_status().AndReturn([])
-        eventlet.sleep(1).AndReturn(None)
         wc.WaitConditionHandle.get_status().AndReturn(['SUCCESS'])
-        eventlet.sleep(1).AndReturn(None)
         wc.WaitConditionHandle.get_status().AndReturn(['SUCCESS', 'SUCCESS'])
-        eventlet.sleep(1).AndReturn(None)
         wc.WaitConditionHandle.get_status().AndReturn(['SUCCESS', 'SUCCESS',
                                                        'SUCCESS'])
 
@@ -184,232 +178,229 @@ class WaitConditionTest(unittest.TestCase):
 
         self.stack.create()
 
-        resource = self.stack.resources['WaitForTheHandle']
-        self.assertEqual(resource.state,
-                         'CREATE_COMPLETE')
+        rsrc = self.stack.resources['WaitForTheHandle']
+        self.assertEqual(rsrc.state,
+                         (rsrc.CREATE, rsrc.COMPLETE))
 
         r = db_api.resource_get_by_name_and_stack(None, 'WaitHandle',
                                                   self.stack.id)
         self.assertEqual(r.name, 'WaitHandle')
         self.m.VerifyAll()
 
-    @stack_delete_after
+    @utils.stack_delete_after
     def test_post_failure_to_handle_count(self):
         self.stack = self.create_stack(template=test_template_wc_count)
-        wc.WaitCondition._create_timeout().AndReturn(eventlet.Timeout(5))
         wc.WaitConditionHandle.get_status().AndReturn([])
-        eventlet.sleep(1).AndReturn(None)
         wc.WaitConditionHandle.get_status().AndReturn(['SUCCESS'])
-        eventlet.sleep(1).AndReturn(None)
         wc.WaitConditionHandle.get_status().AndReturn(['SUCCESS', 'FAILURE'])
 
         self.m.ReplayAll()
 
         self.stack.create()
 
-        resource = self.stack.resources['WaitForTheHandle']
-        self.assertEqual(resource.state,
-                         'CREATE_FAILED')
+        rsrc = self.stack.resources['WaitForTheHandle']
+        self.assertEqual(rsrc.state, (rsrc.CREATE, rsrc.FAILED))
+        reason = rsrc.status_reason
+        self.assertTrue(reason.startswith('WaitConditionFailure:'))
 
         r = db_api.resource_get_by_name_and_stack(None, 'WaitHandle',
                                                   self.stack.id)
         self.assertEqual(r.name, 'WaitHandle')
         self.m.VerifyAll()
 
-    @stack_delete_after
+    @utils.stack_delete_after
     def test_timeout(self):
+        st = time.time()
+
         self.stack = self.create_stack()
-        tmo = eventlet.Timeout(6)
-        wc.WaitCondition._create_timeout().AndReturn(tmo)
+
+        # Avoid the stack create exercising the timeout code at the same time
+        self.m.StubOutWithMock(self.stack, 'timeout_secs')
+        self.stack.timeout_secs().MultipleTimes().AndReturn(None)
+
+        self.m.StubOutWithMock(scheduler, 'wallclock')
+
+        scheduler.wallclock().AndReturn(st)
+        scheduler.wallclock().AndReturn(st + 0.001)
+        scheduler.wallclock().AndReturn(st + 0.1)
         wc.WaitConditionHandle.get_status().AndReturn([])
-        eventlet.sleep(1).AndReturn(None)
+        scheduler.wallclock().AndReturn(st + 4.1)
         wc.WaitConditionHandle.get_status().AndReturn([])
-        eventlet.sleep(1).AndRaise(tmo)
+        scheduler.wallclock().AndReturn(st + 5.1)
 
         self.m.ReplayAll()
 
         self.stack.create()
 
-        resource = self.stack.resources['WaitForTheHandle']
+        rsrc = self.stack.resources['WaitForTheHandle']
 
-        self.assertEqual(resource.state,
-                         'CREATE_FAILED')
-        self.assertEqual(wc.WaitCondition.UPDATE_REPLACE,
-                         resource.handle_update({}))
+        self.assertEqual(rsrc.state, (rsrc.CREATE, rsrc.FAILED))
+        reason = rsrc.status_reason
+        self.assertTrue(reason.startswith('WaitConditionTimeout:'))
+
+        self.assertRaises(resource.UpdateReplace,
+                          rsrc.handle_update, {}, {}, {})
         self.m.VerifyAll()
 
-    @stack_delete_after
+    @utils.stack_delete_after
     def test_FnGetAtt(self):
         self.stack = self.create_stack()
-        wc.WaitCondition._create_timeout().AndReturn(eventlet.Timeout(5))
         wc.WaitConditionHandle.get_status().AndReturn(['SUCCESS'])
 
         self.m.ReplayAll()
         self.stack.create()
 
-        resource = self.stack.resources['WaitForTheHandle']
-        self.assertEqual(resource.state, 'CREATE_COMPLETE')
+        rsrc = self.stack.resources['WaitForTheHandle']
+        self.assertEqual(rsrc.state, (rsrc.CREATE, rsrc.COMPLETE))
 
-        wc_att = resource.FnGetAtt('Data')
+        wc_att = rsrc.FnGetAtt('Data')
         self.assertEqual(wc_att, unicode({}))
 
         handle = self.stack.resources['WaitHandle']
-        self.assertEqual(handle.state, 'CREATE_COMPLETE')
+        self.assertEqual(handle.state, (rsrc.CREATE, rsrc.COMPLETE))
 
         test_metadata = {'Data': 'foo', 'Reason': 'bar',
                          'Status': 'SUCCESS', 'UniqueId': '123'}
         handle.metadata_update(new_metadata=test_metadata)
-        wc_att = resource.FnGetAtt('Data')
+        wc_att = rsrc.FnGetAtt('Data')
         self.assertEqual(wc_att, '{"123": "foo"}')
 
         test_metadata = {'Data': 'dog', 'Reason': 'cat',
                          'Status': 'SUCCESS', 'UniqueId': '456'}
         handle.metadata_update(new_metadata=test_metadata)
-        wc_att = resource.FnGetAtt('Data')
+        wc_att = rsrc.FnGetAtt('Data')
         self.assertEqual(wc_att, u'{"123": "foo", "456": "dog"}')
         self.m.VerifyAll()
 
-    @stack_delete_after
+    @utils.stack_delete_after
     def test_validate_handle_url_bad_stackid(self):
-        # Stub out the stack ID so we have a known value
-        stack_id = 'STACKABCD1234'
-        self.m.StubOutWithMock(uuid, 'uuid4')
-        uuid.uuid4().AndReturn(stack_id)
         self.m.ReplayAll()
 
+        stack_id = 'STACK_HUBSID_1234'
         t = json.loads(test_template_waitcondition)
-        badhandle = ("http://127.0.0.1:8000/v1/waitcondition/" +
+        badhandle = ("http://server.test:8000/v1/waitcondition/" +
                      "arn%3Aopenstack%3Aheat%3A%3Atest_tenant" +
                      "%3Astacks%2Ftest_stack%2F" +
                      "bad1" +
                      "%2Fresources%2FWaitHandle")
         t['Resources']['WaitForTheHandle']['Properties']['Handle'] = badhandle
-        self.stack = self.create_stack(template=json.dumps(t), stub=False)
+        self.stack = self.create_stack(template=json.dumps(t), stub=False,
+                                       stack_id=stack_id)
         self.m.ReplayAll()
 
-        resource = self.stack.resources['WaitForTheHandle']
-        self.assertRaises(ValueError, resource.handle_create)
+        rsrc = self.stack.resources['WaitForTheHandle']
+        self.assertRaises(ValueError, rsrc.handle_create)
 
         self.m.VerifyAll()
 
-    @stack_delete_after
+    @utils.stack_delete_after
     def test_validate_handle_url_bad_stackname(self):
-        # Stub out the stack ID so we have a known value
-        stack_id = 'STACKABCD1234'
-        self.m.StubOutWithMock(uuid, 'uuid4')
-        uuid.uuid4().AndReturn(stack_id)
         self.m.ReplayAll()
 
+        stack_id = 'STACKABCD1234'
         t = json.loads(test_template_waitcondition)
-        badhandle = ("http://127.0.0.1:8000/v1/waitcondition/" +
+        badhandle = ("http://server.test:8000/v1/waitcondition/" +
                      "arn%3Aopenstack%3Aheat%3A%3Atest_tenant" +
                      "%3Astacks%2FBAD_stack%2F" +
                      stack_id + "%2Fresources%2FWaitHandle")
         t['Resources']['WaitForTheHandle']['Properties']['Handle'] = badhandle
-        self.stack = self.create_stack(template=json.dumps(t), stub=False)
+        self.stack = self.create_stack(template=json.dumps(t), stub=False,
+                                       stack_id=stack_id)
 
-        resource = self.stack.resources['WaitForTheHandle']
-        self.assertRaises(ValueError, resource.handle_create)
+        rsrc = self.stack.resources['WaitForTheHandle']
+        self.assertRaises(ValueError, rsrc.handle_create)
 
         self.m.VerifyAll()
 
-    @stack_delete_after
+    @utils.stack_delete_after
     def test_validate_handle_url_bad_tenant(self):
-        # Stub out the stack ID so we have a known value
-        stack_id = 'STACKABCD1234'
-        self.m.StubOutWithMock(uuid, 'uuid4')
-        uuid.uuid4().AndReturn(stack_id)
         self.m.ReplayAll()
 
+        stack_id = 'STACKABCD1234'
         t = json.loads(test_template_waitcondition)
-        badhandle = ("http://127.0.0.1:8000/v1/waitcondition/" +
+        badhandle = ("http://server.test:8000/v1/waitcondition/" +
                      "arn%3Aopenstack%3Aheat%3A%3ABAD_tenant" +
                      "%3Astacks%2Ftest_stack%2F" +
                      stack_id + "%2Fresources%2FWaitHandle")
         t['Resources']['WaitForTheHandle']['Properties']['Handle'] = badhandle
-        self.stack = self.create_stack(template=json.dumps(t), stub=False)
+        self.stack = self.create_stack(stack_id=stack_id,
+                                       template=json.dumps(t), stub=False)
 
-        resource = self.stack.resources['WaitForTheHandle']
-        self.assertRaises(ValueError, resource.handle_create)
+        rsrc = self.stack.resources['WaitForTheHandle']
+        self.assertRaises(ValueError, rsrc.handle_create)
 
         self.m.VerifyAll()
 
-    @stack_delete_after
+    @utils.stack_delete_after
     def test_validate_handle_url_bad_resource(self):
-        # Stub out the stack ID so we have a known value
-        stack_id = 'STACKABCD1234'
-        self.m.StubOutWithMock(uuid, 'uuid4')
-        uuid.uuid4().AndReturn(stack_id)
         self.m.ReplayAll()
 
+        stack_id = 'STACK_HUBR_1234'
         t = json.loads(test_template_waitcondition)
-        badhandle = ("http://127.0.0.1:8000/v1/waitcondition/" +
+        badhandle = ("http://server.test:8000/v1/waitcondition/" +
                      "arn%3Aopenstack%3Aheat%3A%3Atest_tenant" +
                      "%3Astacks%2Ftest_stack%2F" +
                      stack_id + "%2Fresources%2FBADHandle")
         t['Resources']['WaitForTheHandle']['Properties']['Handle'] = badhandle
-        self.stack = self.create_stack(template=json.dumps(t), stub=False)
+        self.stack = self.create_stack(stack_id=stack_id,
+                                       template=json.dumps(t), stub=False)
 
-        resource = self.stack.resources['WaitForTheHandle']
-        self.assertRaises(ValueError, resource.handle_create)
+        rsrc = self.stack.resources['WaitForTheHandle']
+        self.assertRaises(ValueError, rsrc.handle_create)
 
         self.m.VerifyAll()
 
-    @stack_delete_after
+    @utils.stack_delete_after
     def test_validate_handle_url_bad_resource_type(self):
-        # Stub out the stack ID so we have a known value
-        stack_id = 'STACKABCD1234'
-        self.m.StubOutWithMock(uuid, 'uuid4')
-        uuid.uuid4().AndReturn(stack_id)
         self.m.ReplayAll()
-
+        stack_id = 'STACKABCD1234'
         t = json.loads(test_template_waitcondition)
-        badhandle = ("http://127.0.0.1:8000/v1/waitcondition/" +
+        badhandle = ("http://server.test:8000/v1/waitcondition/" +
                      "arn%3Aopenstack%3Aheat%3A%3Atest_tenant" +
                      "%3Astacks%2Ftest_stack%2F" +
                      stack_id + "%2Fresources%2FWaitForTheHandle")
         t['Resources']['WaitForTheHandle']['Properties']['Handle'] = badhandle
-        self.stack = self.create_stack(template=json.dumps(t), stub=False)
+        self.stack = self.create_stack(stack_id=stack_id,
+                                       template=json.dumps(t), stub=False)
 
-        resource = self.stack.resources['WaitForTheHandle']
-        self.assertRaises(ValueError, resource.handle_create)
+        rsrc = self.stack.resources['WaitForTheHandle']
+        self.assertRaises(ValueError, rsrc.handle_create)
 
         self.m.VerifyAll()
 
 
-@attr(tag=['unit', 'resource', 'WaitConditionHandle'])
-@attr(speed='fast')
-class WaitConditionHandleTest(unittest.TestCase):
+class WaitConditionHandleTest(HeatTestCase):
     def setUp(self):
-        self.m = mox.Mox()
+        super(WaitConditionHandleTest, self).setUp()
         cfg.CONF.set_default('heat_waitcondition_server_url',
-                             'http://127.0.0.1:8000/v1/waitcondition')
+                             'http://server.test:8000/v1/waitcondition')
 
         self.fc = fakes.FakeKeystoneClient()
-        self.stack = self.create_stack()
+        utils.setup_dummy_db()
 
     def tearDown(self):
-        self.m.UnsetStubs()
+        super(WaitConditionHandleTest, self).tearDown()
+        utils.reset_dummy_db()
 
-    def create_stack(self, stack_name='test_stack2', params={}):
+    def create_stack(self, stack_name=None, stack_id=None):
         temp = template_format.parse(test_template_waitcondition)
         template = parser.Template(temp)
-        parameters = parser.Parameters(stack_name, template, params)
-        ctx = context.get_admin_context()
-        ctx.tenant_id = 'test_tenant'
-        stack = parser.Stack(ctx, stack_name, template, parameters,
+        ctx = utils.dummy_context(tenant_id='test_tenant')
+        if stack_name is None:
+            stack_name = utils.random_name()
+        stack = parser.Stack(ctx, stack_name, template,
                              disable_rollback=True)
         # Stub out the UUID for this test, so we can get an expected signature
-        self.m.StubOutWithMock(uuid, 'uuid4')
-        uuid.uuid4().AndReturn('STACKABCD1234')
-        self.m.ReplayAll()
-        stack.store()
+        if stack_id is not None:
+            with utils.UUIDStub(stack_id):
+                stack.store()
+        else:
+            stack.store()
+        self.stack_id = stack.id
 
         # Stub waitcondition status so all goes CREATE_COMPLETE
         self.m.StubOutWithMock(wc.WaitConditionHandle, 'get_status')
         wc.WaitConditionHandle.get_status().AndReturn(['SUCCESS'])
-        self.m.StubOutWithMock(wc.WaitCondition, '_create_timeout')
-        wc.WaitCondition._create_timeout().AndReturn(eventlet.Timeout(5))
 
         # Stub keystone() with fake client
         self.m.StubOutWithMock(wc.WaitConditionHandle, 'keystone')
@@ -425,131 +416,143 @@ class WaitConditionHandleTest(unittest.TestCase):
 
         return stack
 
-    @stack_delete_after
+    @utils.stack_delete_after
     def test_handle(self):
+        stack_id = 'STACKABCD1234'
+        stack_name = 'test_stack2'
         created_time = datetime.datetime(2012, 11, 29, 13, 49, 37)
+        self.stack = self.create_stack(stack_id=stack_id,
+                                       stack_name=stack_name)
 
-        resource = self.stack.resources['WaitHandle']
-        resource.created_time = created_time
-        self.assertEqual(resource.state, 'CREATE_COMPLETE')
+        rsrc = self.stack.resources['WaitHandle']
+        # clear the url
+        db_api.resource_data_set(rsrc, 'ec2_signed_url', None, False)
+
+        rsrc.created_time = created_time
+        self.assertEqual(rsrc.state, (rsrc.CREATE, rsrc.COMPLETE))
 
         expected_url = "".join([
-            'http://127.0.0.1:8000/v1/waitcondition/',
+            'http://server.test:8000/v1/waitcondition/',
             'arn%3Aopenstack%3Aheat%3A%3Atest_tenant%3Astacks%2F',
-            'test_stack2%2FSTACKABCD1234%2Fresources%2F',
+            'test_stack2%2F', stack_id, '%2Fresources%2F',
             'WaitHandle?',
             'Timestamp=2012-11-29T13%3A49%3A37Z&',
             'SignatureMethod=HmacSHA256&',
             'AWSAccessKeyId=4567&',
             'SignatureVersion=2&',
             'Signature=',
-            'ePyTwmC%2F1kSigeo%2Fha7kP8Avvb45G9Y7WOQWe4F%2BnXM%3D'])
+            'fHyt3XFnHq8%2FSwYaVcHdJka1hz6jdK5mHtgbo8OOKbQ%3D'])
 
-        self.assertEqual(expected_url, resource.FnGetRefId())
+        self.assertEqual(unicode(expected_url), rsrc.FnGetRefId())
 
-        self.assertEqual(resource.UPDATE_REPLACE, resource.handle_update({}))
+        self.assertRaises(resource.UpdateReplace,
+                          rsrc.handle_update, {}, {}, {})
         self.m.VerifyAll()
 
-    @stack_delete_after
+    @utils.stack_delete_after
     def test_metadata_update(self):
-        resource = self.stack.resources['WaitHandle']
-        self.assertEqual(resource.state, 'CREATE_COMPLETE')
+        self.stack = self.create_stack()
+        rsrc = self.stack.resources['WaitHandle']
+        self.assertEqual(rsrc.state, (rsrc.CREATE, rsrc.COMPLETE))
 
         test_metadata = {'Data': 'foo', 'Reason': 'bar',
                          'Status': 'SUCCESS', 'UniqueId': '123'}
-        resource.metadata_update(new_metadata=test_metadata)
+        rsrc.metadata_update(new_metadata=test_metadata)
         handle_metadata = {u'123': {u'Data': u'foo',
                                     u'Reason': u'bar',
                                     u'Status': u'SUCCESS'}}
-        self.assertEqual(resource.metadata, handle_metadata)
+        self.assertEqual(handle_metadata, rsrc.metadata)
         self.m.VerifyAll()
 
-    @stack_delete_after
+    @utils.stack_delete_after
     def test_metadata_update_invalid(self):
-        resource = self.stack.resources['WaitHandle']
-        self.assertEqual(resource.state, 'CREATE_COMPLETE')
+        self.stack = self.create_stack()
+        rsrc = self.stack.resources['WaitHandle']
+        self.assertEqual(rsrc.state, (rsrc.CREATE, rsrc.COMPLETE))
 
         # metadata_update should raise a ValueError if the metadata
         # is missing any of the expected keys
         err_metadata = {'Data': 'foo', 'Status': 'SUCCESS', 'UniqueId': '123'}
-        self.assertRaises(ValueError, resource.metadata_update,
+        self.assertRaises(ValueError, rsrc.metadata_update,
                           new_metadata=err_metadata)
 
         err_metadata = {'Data': 'foo', 'Reason': 'bar', 'UniqueId': '1234'}
-        self.assertRaises(ValueError, resource.metadata_update,
+        self.assertRaises(ValueError, rsrc.metadata_update,
                           new_metadata=err_metadata)
 
         err_metadata = {'Data': 'foo', 'Reason': 'bar', 'UniqueId': '1234'}
-        self.assertRaises(ValueError, resource.metadata_update,
+        self.assertRaises(ValueError, rsrc.metadata_update,
                           new_metadata=err_metadata)
 
         err_metadata = {'data': 'foo', 'reason': 'bar',
                         'status': 'SUCCESS', 'uniqueid': '1234'}
-        self.assertRaises(ValueError, resource.metadata_update,
+        self.assertRaises(ValueError, rsrc.metadata_update,
                           new_metadata=err_metadata)
 
         # Also any Status other than SUCCESS or FAILURE should be rejected
         err_metadata = {'Data': 'foo', 'Reason': 'bar',
                         'Status': 'UCCESS', 'UniqueId': '123'}
-        self.assertRaises(ValueError, resource.metadata_update,
+        self.assertRaises(ValueError, rsrc.metadata_update,
                           new_metadata=err_metadata)
         err_metadata = {'Data': 'foo', 'Reason': 'bar',
                         'Status': 'wibble', 'UniqueId': '123'}
-        self.assertRaises(ValueError, resource.metadata_update,
+        self.assertRaises(ValueError, rsrc.metadata_update,
                           new_metadata=err_metadata)
         err_metadata = {'Data': 'foo', 'Reason': 'bar',
                         'Status': 'success', 'UniqueId': '123'}
-        self.assertRaises(ValueError, resource.metadata_update,
+        self.assertRaises(ValueError, rsrc.metadata_update,
                           new_metadata=err_metadata)
         err_metadata = {'Data': 'foo', 'Reason': 'bar',
                         'Status': 'FAIL', 'UniqueId': '123'}
-        self.assertRaises(ValueError, resource.metadata_update,
+        self.assertRaises(ValueError, rsrc.metadata_update,
                           new_metadata=err_metadata)
         self.m.VerifyAll()
 
-    @stack_delete_after
+    @utils.stack_delete_after
     def test_get_status(self):
-        resource = self.stack.resources['WaitHandle']
-        self.assertEqual(resource.state, 'CREATE_COMPLETE')
+        self.stack = self.create_stack()
+        rsrc = self.stack.resources['WaitHandle']
+        self.assertEqual(rsrc.state, (rsrc.CREATE, rsrc.COMPLETE))
 
         # UnsetStubs, don't want get_status stubbed anymore..
         self.m.VerifyAll()
         self.m.UnsetStubs()
 
-        self.assertEqual(resource.get_status(), [])
+        self.assertEqual(rsrc.get_status(), [])
 
         test_metadata = {'Data': 'foo', 'Reason': 'bar',
                          'Status': 'SUCCESS', 'UniqueId': '123'}
-        resource.metadata_update(new_metadata=test_metadata)
-        self.assertEqual(resource.get_status(), ['SUCCESS'])
+        rsrc.metadata_update(new_metadata=test_metadata)
+        self.assertEqual(rsrc.get_status(), ['SUCCESS'])
 
         test_metadata = {'Data': 'foo', 'Reason': 'bar',
                          'Status': 'SUCCESS', 'UniqueId': '456'}
-        resource.metadata_update(new_metadata=test_metadata)
-        self.assertEqual(resource.get_status(), ['SUCCESS', 'SUCCESS'])
+        rsrc.metadata_update(new_metadata=test_metadata)
+        self.assertEqual(rsrc.get_status(), ['SUCCESS', 'SUCCESS'])
 
         # re-stub keystone() with fake client or stack delete fails
         self.m.StubOutWithMock(wc.WaitConditionHandle, 'keystone')
         wc.WaitConditionHandle.keystone().MultipleTimes().AndReturn(self.fc)
         self.m.ReplayAll()
 
-    @stack_delete_after
+    @utils.stack_delete_after
     def test_get_status_reason(self):
-        resource = self.stack.resources['WaitHandle']
-        self.assertEqual(resource.state, 'CREATE_COMPLETE')
+        self.stack = self.create_stack()
+        rsrc = self.stack.resources['WaitHandle']
+        self.assertEqual(rsrc.state, (rsrc.CREATE, rsrc.COMPLETE))
 
         test_metadata = {'Data': 'foo', 'Reason': 'bar',
                          'Status': 'SUCCESS', 'UniqueId': '123'}
-        resource.metadata_update(new_metadata=test_metadata)
-        self.assertEqual(resource.get_status_reason('SUCCESS'), 'bar')
+        rsrc.metadata_update(new_metadata=test_metadata)
+        self.assertEqual('bar', rsrc.get_status_reason('SUCCESS'))
 
         test_metadata = {'Data': 'dog', 'Reason': 'cat',
                          'Status': 'SUCCESS', 'UniqueId': '456'}
-        resource.metadata_update(new_metadata=test_metadata)
-        self.assertEqual(resource.get_status_reason('SUCCESS'), 'bar;cat')
+        rsrc.metadata_update(new_metadata=test_metadata)
+        self.assertEqual('bar;cat', rsrc.get_status_reason('SUCCESS'))
 
         test_metadata = {'Data': 'boo', 'Reason': 'hoo',
                          'Status': 'FAILURE', 'UniqueId': '789'}
-        resource.metadata_update(new_metadata=test_metadata)
-        self.assertEqual(resource.get_status_reason('FAILURE'), 'hoo')
+        rsrc.metadata_update(new_metadata=test_metadata)
+        self.assertEqual('hoo', rsrc.get_status_reason('FAILURE'))
         self.m.VerifyAll()
