@@ -1,5 +1,5 @@
 # Copyright (c) 2011 X.commerce, a business unit of eBay Inc.
-# Copyright 2011 OpenStack, LLC
+# Copyright 2011 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -113,12 +113,16 @@ class FakeHTTPClient(base_client.HTTPClient):
         return (200, {"servers": [
             {'id': 1234, 'name': 'sample-server'},
             {'id': 5678, 'name': 'sample-server2'},
+            {'id': 9101, 'name': 'hard-reboot'},
+            {'id': 9102, 'name': 'server-with-no-ip'},
             {'id': 9999, 'name': 'sample-server3'}
         ]})
 
     def get_servers_detail(self, **kw):
         return (200, {"servers": [{"id": 1234,
                                    "name": "sample-server",
+                                   "OS-EXT-SRV-ATTR:instance_name":
+                                   "sample-server",
                                    "image": {"id": 2,
                                              "name": "sample image"},
                                    "flavor": {"id": 1,
@@ -135,10 +139,14 @@ class FakeHTTPClient(base_client.HTTPClient):
                                                              "5.6.7.8"}],
                                    "private": [{"version": 4,
                                                 "addr": "10.11.12.13"}]},
+                                   "accessIPv4": "",
+                                   "accessIPv6": "",
                                    "metadata": {"Server Label": "Web Head 1",
                                                 "Image Version": "2.1"}},
                                   {"id": 5678,
                                    "name": "sample-server2",
+                                   "OS-EXT-SRV-ATTR:instance_name":
+                                   "sample-server2",
                                    "image": {"id": 2,
                                              "name": "sample image"},
                                    "flavor": {"id": 1,
@@ -146,6 +154,8 @@ class FakeHTTPClient(base_client.HTTPClient):
                                    "hostId":
                                    "9e107d9d372bb6826bd81d3542a419d6",
                                    "status": "ACTIVE",
+                                   "accessIPv4": "192.0.2.0",
+                                   "accessIPv6": "::babe:4317:0A83",
                                    "addresses": {"public": [{"version": 4,
                                                              "addr":
                                                              "4.5.6.7"},
@@ -155,8 +165,48 @@ class FakeHTTPClient(base_client.HTTPClient):
                                    "private": [{"version": 4,
                                                 "addr": "10.13.12.13"}]},
                                    "metadata": {"Server Label": "DB 1"}},
+                                  {"id": 9101,
+                                   "name": "hard-reboot",
+                                   "OS-EXT-SRV-ATTR:instance_name":
+                                   "hard-reboot",
+                                   "image": {"id": 2,
+                                             "name": "sample image"},
+                                   "flavor": {"id": 1,
+                                              "name": "256 MB Server"},
+                                   "hostId":
+                                   "9e44d8d435c43dd8d96bb63ed995605f",
+                                   "status": "HARD_REBOOT",
+                                   "accessIPv4": "",
+                                   "accessIPv6": "",
+                                   "addresses": {"public": [{"version": 4,
+                                                             "addr":
+                                                             "172.17.1.2"},
+                                                            {"version": 4,
+                                                             "addr":
+                                                             "10.20.30.40"}],
+                                   "private": [{"version": 4,
+                                                "addr": "10.13.12.13"}]},
+                                   "metadata": {"Server Label": "DB 1"}},
+                                  {"id": 9102,
+                                   "name": "server-with-no-ip",
+                                   "OS-EXT-SRV-ATTR:instance_name":
+                                   "server-with-no-ip",
+                                   "image": {"id": 2,
+                                             "name": "sample image"},
+                                   "flavor": {"id": 1,
+                                              "name": "256 MB Server"},
+                                   "hostId":
+                                   "c1365ba78c624df9b2ff446515a682f5",
+                                   "status": "ACTIVE",
+                                   "accessIPv4": "",
+                                   "accessIPv6": "",
+                                   "addresses": {
+                                       "empty_net": []},
+                                   "metadata": {"Server Label": "DB 1"}},
                                   {"id": 9999,
                                    "name": "sample-server3",
+                                   "OS-EXT-SRV-ATTR:instance_name":
+                                   "sample-server3",
                                    "image": {"id": 3,
                                              "name": "sample image"},
                                    "flavor": {"id": 3,
@@ -164,6 +214,8 @@ class FakeHTTPClient(base_client.HTTPClient):
                                    "hostId":
                                    "9e107d9d372bb6826bd81d3542a419d6",
                                    "status": "ACTIVE",
+                                   "accessIPv4": "",
+                                   "accessIPv6": "",
                                    "addresses": {
                                    "public": [{"version": 4,
                                                "addr": "4.5.6.7"},
@@ -196,7 +248,7 @@ class FakeHTTPClient(base_client.HTTPClient):
         return (200, r)
 
     def get_servers_WikiServerOne2(self, **kw):
-        r = {'server': self.get_servers_detail()[1]['servers'][0]}
+        r = {'server': self.get_servers_detail()[1]['servers'][3]}
         return (200, r)
 
     def get_servers_5678(self, **kw):
@@ -259,6 +311,10 @@ class FakeHTTPClient(base_client.HTTPClient):
     def get_servers_9999_diagnostics(self, **kw):
         return (200, 'Fake diagnostics')
 
+    def get_servers_9102(self, **kw):
+        r = {'server': self.get_servers_detail()[1]['servers'][3]}
+        return (200, r)
+
     def get_servers_1234_actions(self, **kw):
         return (200, {'actions': [{'action': 'rebuild',
                                    'error': None,
@@ -322,6 +378,10 @@ class FakeHTTPClient(base_client.HTTPClient):
             assert body[action] is None
         elif action == 'unlock':
             assert body[action] is None
+        elif action == 'suspend':
+            assert body[action] is None
+        elif action == 'resume':
+            assert body[action] is None
         elif action == 'addFixedIp':
             assert body[action].keys() == ['networkId']
         elif action == 'removeFixedIp':
@@ -821,3 +881,6 @@ class FakeHTTPClient(base_client.HTTPClient):
         result = {'host': 'dummy'}
         result.update(body)
         return (200, result)
+
+    def get_os_availability_zone(self, *kw):
+        return (200, {"availabilityZoneInfo": [{'zoneName': 'nova1'}]})
